@@ -12,7 +12,7 @@ function ViewModel() {
     self.isRestarting = ko.observable(false);
     self.useGlobalOptions = ko.observable(true).extend({ persist: 'useGlobalOptions' });
     self.refreshRate = ko.observable(1).extend({ persist: 'pageRefreshRate' });
-    self.dateFormat = ko.observable('DD/MM/YYYY HH:mm').extend({ persist: 'pageDateFormat' });
+    self.dateFormat = ko.observable('fromNow').extend({ persist: 'pageDateFormat' });
     self.displayTabbed = ko.observable().extend({ persist: 'displayTabbed' });
     self.displayCompact = ko.observable(false).extend({ persist: 'displayCompact' });
     self.confirmDeleteQueue = ko.observable(true).extend({ persist: 'confirmDeleteQueue' });
@@ -40,6 +40,8 @@ function ViewModel() {
     self.quotaLimit = ko.observable();
     self.quotaLimitLeft = ko.observable();
     self.systemLoad = ko.observable();
+    self.cacheSize = ko.observable();
+    self.cacheArticles = ko.observable();
     self.nrWarnings = ko.observable(0);
     self.allWarnings = ko.observableArray([]);
     self.allMessages = ko.observableArray([]);
@@ -48,7 +50,7 @@ function ViewModel() {
 
     // Statusinfo container
     self.hasStatusInfo = ko.observable(false);
-    self.hasDiskStatusInfo = ko.observable(false);
+    self.hasPerformanceInfo = ko.observable(false);
     self.statusInfo = {};
     self.statusInfo.folders = ko.observableArray([]);
     self.statusInfo.servers = ko.observableArray([]);
@@ -59,8 +61,6 @@ function ViewModel() {
     self.statusInfo.pystone = ko.observable();
     self.statusInfo.cpumodel = ko.observable();
     self.statusInfo.loglevel = ko.observable();
-    self.statusInfo.cache_size = ko.observable();
-    self.statusInfo.cache_art = ko.observable();
     self.statusInfo.downloaddir = ko.observable();
     self.statusInfo.downloaddirspeed = ko.observable();
     self.statusInfo.completedir = ko.observable();
@@ -173,8 +173,8 @@ function ViewModel() {
         }
 
         // Did we exceed the space?
-        self.diskSpaceExceeded1(parseInt(response.queue.mbleft)/1024 > parseInt(response.queue.diskspace1))
-        self.diskSpaceExceeded2(parseInt(response.queue.mbleft)/1024 > parseInt(response.queue.diskspace2))
+        self.diskSpaceExceeded1(parseInt(response.queue.mbleft)/1024 > parseFloat(response.queue.diskspace1))
+        self.diskSpaceExceeded2(parseInt(response.queue.mbleft)/1024 > parseFloat(response.queue.diskspace2))
 
         // Quota
         self.quotaLimit(response.queue.quota)
@@ -183,6 +183,10 @@ function ViewModel() {
         // System load
         self.systemLoad(response.queue.loadavg)
 
+        // Cache
+        self.cacheSize(response.queue.cache_size)
+        self.cacheArticles(response.queue.cache_art)
+
         // Warnings (new warnings will trigger an update of allMessages)
         self.nrWarnings(response.queue.have_warnings)
 
@@ -327,6 +331,13 @@ function ViewModel() {
                 // Split title & speed
                 var dataSplit = data.split('|||');
 
+                // Maybe the result is actually the login page?
+                if(dataSplit[0].substring(0, 11) === '<html lang=') {
+                    // Redirect
+                    document.location = document.location
+                    return
+                }
+
                 // Set title
                 self.title(dataSplit[0]);
 
@@ -467,6 +478,12 @@ function ViewModel() {
             return;
         }
 
+        // Fix DateJS bug it has some strange problem with the current day-of-month + 1
+        // Removing the space makes DateJS work properly
+        newValue = newValue.replace(/\s*h|\s*m|\s*d/g, function(match) {
+            return match.trim()
+        });
+
         // Parse
         var pauseParsed = Date.parse(newValue);
 
@@ -519,6 +536,7 @@ function ViewModel() {
         callAPI({
             mode: 'warnings'
         }).then(function(response) {
+
             // Reset it all
             self.allWarnings.removeAll();
             if(response) {
@@ -527,20 +545,16 @@ function ViewModel() {
 
                 // Go over all warnings and add
                 $.each(response.warnings, function(index, warning) {
-                    // Split warning into parts
-                    var warningSplit = warning.split(/\n/);
-
                     // Reformat CSS label and date
+                    // Replaces spaces by non-breakable spaces and newlines with br's
                     var warningData = {
                         index: index,
-                        type: glitterTranslate.status[warningSplit[1]].slice(0, -1),
-                        text: warningSplit.slice(2).join('<br/>').replace(/ /g, '\u00A0'), // Recombine if multiple lines
-                        date: displayDateTime(warningSplit[0], self.dateFormat(), 'YYYY-MM-DD HH:mm'),
-                        timestamp: moment(warningSplit[0], 'YYYY-MM-DD HH:mm').unix(),
-                        css: (warningSplit[1] == "ERROR" ? "danger" : warningSplit[1] == "WARNING" ? "warning" : "info"),
+                        type: glitterTranslate.status[warning.type].slice(0, -1),
+                        text: convertHTMLtoText(warning.text).replace(/ /g, '\u00A0').replace(/(?:\r\n|\r|\n)/g, '<br />'),
+                        timestamp: warning.time,
+                        css: (warning.type == "ERROR" ? "danger" : warning.type == "WARNING" ? "warning" : "info"),
                         clear: self.clearWarnings
                     };
-
                     self.allWarnings.push(warningData)
                 })
             }
@@ -589,7 +603,7 @@ function ViewModel() {
     }
 
     // Shutdown options
-    self.onQueueFinish.subscribe(function(newValue) {
+    self.setOnQueueFinish = function(model, event) {
         // Ignore updates before the page is done
         if(!self.hasStatusInfo()) return;
 
@@ -597,9 +611,12 @@ function ViewModel() {
         callAPI({
             mode: 'queue',
             name: 'change_complete_action',
-            value: newValue
+            value: $(event.target).val()
         })
-    })
+
+        // Top stop blinking while the API is calling
+        self.onQueueFinish($(event.target).val())
+    }
 
     // Use global settings or device-specific?
     self.useGlobalOptions.subscribe(function(newValue) {
@@ -749,8 +766,6 @@ function ViewModel() {
         callAPI({ mode: 'fullstatus', skip_dashboard: (!statusFullRefresh)*1 }).then(function(data) {
             // Update basic
             self.statusInfo.loglevel(data.status.loglevel)
-            self.statusInfo.cache_art(data.status.cache_art)
-            self.statusInfo.cache_size(data.status.cache_size)
             self.statusInfo.folders(data.status.folders)
 
             // Update the full set
@@ -766,19 +781,32 @@ function ViewModel() {
                 self.statusInfo.publicipv4(data.status.publicipv4)
                 self.statusInfo.ipv6(data.status.ipv6 || glitterTranslate.noneText)
                 // Loaded disk info
-                self.hasDiskStatusInfo(true)
+                self.hasPerformanceInfo(true)
             }
 
             // Update the servers
-            if(self.statusInfo.servers().length == 0) {
+            if(self.statusInfo.servers().length != data.status.servers.length) {
+                // Only now we can subscribe to the log-level-changes! (only at start)
+                if(self.statusInfo.servers().length == 0) {
+                    self.statusInfo.loglevel.subscribe(function(newValue) {
+                        // Update log-level
+                        callSpecialAPI('./status/change_loglevel/', {
+                            loglevel: newValue
+                        });
+                    })
+                }
+
+                // Empty them, in case of update
+                self.statusInfo.servers([])
+
                 // Initial add
                 $.each(data.status.servers, function() {
                     self.statusInfo.servers.push({
-                        'servername': this.servername,
-                        'serveroptional': this.serveroptional,
-                        'serverpriority': this.serverpriority,
-                        'servertotalconn': this.servertotalconn,
-                        'serverssl': this.serverssl,
+                        'servername': ko.observable(this.servername),
+                        'serveroptional': ko.observable(this.serveroptional),
+                        'serverpriority': ko.observable(this.serverpriority),
+                        'servertotalconn': ko.observable(this.servertotalconn),
+                        'serverssl': ko.observable(this.serverssl),
                         'serversslinfo': ko.observable(this.serversslinfo),
                         'serveractiveconn': ko.observable(this.serveractiveconn),
                         'servererror': ko.observable(this.servererror),
@@ -786,23 +814,20 @@ function ViewModel() {
                         'serverconnections': ko.observableArray(this.serverconnections)
                     })
                 })
-
-                // Only now we can subscribe to the log-level-changes!
-                self.statusInfo.loglevel.subscribe(function(newValue) {
-                    // Update log-level
-                    callSpecialAPI('./status/change_loglevel/', {
-                        loglevel: newValue
-                    });
-                })
             } else {
                 // Update
                 $.each(data.status.servers, function(index) {
                     var activeServer = self.statusInfo.servers()[index];
-                    activeServer.serveractiveconn(this.serveractiveconn)
-                    activeServer.servererror(this.servererror)
-                    activeServer.serveractive(this.serveractive)
+                    activeServer.servername(this.servername),
+                    activeServer.serveroptional(this.serveroptional),
+                    activeServer.serverpriority(this.serverpriority),
+                    activeServer.servertotalconn(this.servertotalconn),
+                    activeServer.serverssl(this.serverssl),
+                    activeServer.serversslinfo(this.serversslinfo),
+                    activeServer.serveractiveconn(this.serveractiveconn),
+                    activeServer.servererror(this.servererror),
+                    activeServer.serveractive(this.serveractive),
                     activeServer.serverconnections(this.serverconnections)
-                    activeServer.serversslinfo(this.serversslinfo)
                 })
             }
 
@@ -816,7 +841,7 @@ function ViewModel() {
 
     // Do a disk-speedtest
     self.testDiskSpeed = function(item, event) {
-        self.hasDiskStatusInfo(false)
+        self.hasPerformanceInfo(false)
 
         // Run it and then display it
         callSpecialAPI('./status/dashrefresh/').then(function() {
@@ -824,6 +849,24 @@ function ViewModel() {
         })
     }
 
+    // Download a test-NZB
+    self.testDownload = function(data, event) {
+        var nzbSize = $(event.target).data('size')
+        // Build request
+        var theCall = {
+            mode: "addurl",
+            name: "https://sabnzbd.org/tests/test_download_" + nzbSize + ".nzb",
+            priority: self.queue.priorityName["Force"]
+        }
+
+        // Add
+        callAPI(theCall).then(function(r) {
+            // Hide and reset/refresh
+            self.refresh()
+            $("#modal-options").modal("hide");
+        });
+    }
+
     // Unblock server
     self.unblockServer = function(servername) {
         callSpecialAPI("./status/unblock_server/", {
