@@ -1,5 +1,5 @@
 #!/usr/bin/python -OO
-# Copyright 2008-2017 The SABnzbd-Team <team@sabnzbd.org>
+# Copyright 2007-2019 The SABnzbd-Team <team@sabnzbd.org>
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -24,9 +24,12 @@ import re
 import logging
 import threading
 import shutil
+import time
+import random
+from hashlib import md5
+from urlparse import urlparse
 import sabnzbd.misc
 from sabnzbd.constants import CONFIG_VERSION, NORMAL_PRIORITY, DEFAULT_PRIORITY, MAX_WIN_DFOLDER
-from sabnzbd.utils import listquote
 from sabnzbd.utils import configobj
 from sabnzbd.decorators import synchronized
 
@@ -43,6 +46,8 @@ database = {}               # Holds the option dictionary
 modified = False            # Signals a change in option dictionary
                             # Should be reset after saving to settings file
 
+paramfinder = re.compile(r'''(?:'.*?')|(?:".*?")|(?:[^'",\s][^,]*)''')
+
 
 class Option(object):
     """ Basic option class, basic fields """
@@ -260,7 +265,7 @@ class OptionList(Option):
                 if '"' not in value and ',' not in value:
                     value = value.split()
                 else:
-                    value = listquote.simplelist(value)
+                    value = paramfinder.findall(value)
             if self.__validation:
                 error, value = self.__validation(value)
             if not error:
@@ -275,6 +280,14 @@ class OptionList(Option):
         else:
             return ', '.join(lst)
 
+    def default_string(self):
+        """ Return the default list as a comma-separated string """
+        lst = self.default()
+        if isinstance(lst, basestring):
+            return lst
+        else:
+            return ', '.join(lst)
+
 
 class OptionStr(Option):
     """ String class """
@@ -372,20 +385,18 @@ class ConfigServer(object):
         self.displayname = OptionStr(name, 'displayname', '', add=False)
         self.host = OptionStr(name, 'host', '', add=False)
         self.port = OptionNumber(name, 'port', 119, 0, 2 ** 16 - 1, add=False)
-        self.timeout = OptionNumber(name, 'timeout', 120, 30, 240, add=False)
+        self.timeout = OptionNumber(name, 'timeout', 60, 20, 240, add=False)
         self.username = OptionStr(name, 'username', '', add=False)
         self.password = OptionPassword(name, 'password', '', add=False)
         self.connections = OptionNumber(name, 'connections', 1, 0, 100, add=False)
         self.ssl = OptionBool(name, 'ssl', False, add=False)
-        self.ssl_verify = OptionNumber(name, 'ssl_verify', 1, add=False) # 0=No, 1=Normal, 2=Strict (hostname verification)
+        self.ssl_verify = OptionNumber(name, 'ssl_verify', 2, add=False)  # 0=No, 1=Normal, 2=Strict (hostname verification)
+        self.ssl_ciphers = OptionStr(name, 'ssl_ciphers', '', add=False)
         self.enable = OptionBool(name, 'enable', True, add=False)
         self.optional = OptionBool(name, 'optional', False, add=False)
         self.retention = OptionNumber(name, 'retention', add=False)
         self.send_group = OptionBool(name, 'send_group', False, add=False)
-        self.priority = OptionNumber(name, 'priority', 0, 0, 100, add=False)
-        # 'fillserver' field only here in order to set a proper priority when converting
-        self.fillserver = OptionBool(name, 'fillserver', False, add=False)
-        self.categories = OptionList(name, 'categories', default_val=['Default'], add=False)
+        self.priority = OptionNumber(name, 'priority', 0, 0, 99, add=False)
         self.notes = OptionStr(name, 'notes', '', add=False)
 
         self.set_dict(values)
@@ -393,15 +404,15 @@ class ConfigServer(object):
 
     def set_dict(self, values):
         """ Set one or more fields, passed as dictionary """
-        for kw in ('displayname', 'host', 'port', 'timeout', 'username', 'password', 'connections', 'fillserver',
-                   'ssl', 'ssl_verify', 'send_group', 'enable', 'optional', 'retention', 'priority', 'categories', 'notes'):
+        for kw in ('displayname', 'host', 'port', 'timeout', 'username', 'password', 'connections', 'ssl',
+                   'ssl_verify', 'ssl_ciphers', 'send_group', 'enable', 'optional', 'retention', 'priority', 'notes'):
             try:
                 value = values[kw]
             except KeyError:
                 continue
             exec 'self.%s.set(value)' % kw
-            if not self.displayname():
-                self.displayname.set(self.__name)
+        if not self.displayname():
+            self.displayname.set(self.__name)
         return True
 
     def get_dict(self, safe=False):
@@ -420,12 +431,12 @@ class ConfigServer(object):
         dict['connections'] = self.connections()
         dict['ssl'] = self.ssl()
         dict['ssl_verify'] = self.ssl_verify()
+        dict['ssl_ciphers'] = self.ssl_ciphers()
         dict['enable'] = self.enable()
         dict['optional'] = self.optional()
         dict['retention'] = self.retention()
         dict['send_group'] = self.send_group()
         dict['priority'] = self.priority()
-        dict['categories'] = self.categories()
         dict['notes'] = self.notes()
         return dict
 
@@ -452,7 +463,7 @@ class ConfigCat(object):
         self.pp = OptionStr(name, 'pp', '', add=False)
         self.script = OptionStr(name, 'script', 'Default', add=False)
         self.dir = OptionDir(name, 'dir', add=False, create=False)
-        self.newzbin = OptionList(name, 'newzbin', add=False)
+        self.newzbin = OptionList(name, 'newzbin', add=False, validation=validate_single_tag)
         self.priority = OptionNumber(name, 'priority', DEFAULT_PRIORITY, add=False)
 
         self.set_dict(values)
@@ -542,7 +553,7 @@ class OptionFilters(Option):
                 if isinstance(val, list):
                     filters.append(val)
                 else:
-                    filters.append(listquote.simplelist(val))
+                    filters.append(paramfinder.findall(val))
                 while len(filters[-1]) < 7:
                     filters[-1].append('1')
                 if not filters[-1][6]:
@@ -756,9 +767,6 @@ def _read_config(path, try_backup=False):
     CFG['__encoding__'] = u'utf-8'
     CFG['__version__'] = unicode(CONFIG_VERSION)
 
-    if 'misc' in CFG:
-        compatibility_fix(CFG['misc'])
-
     # Use CFG data to set values for all static options
     for section in database:
         if section not in ('servers', 'categories', 'rss'):
@@ -782,7 +790,6 @@ def _read_config(path, try_backup=False):
 def save_config(force=False):
     """ Update Setup file with current option values """
     global CFG, database, modified
-    if 0: assert isinstance(CFG, configobj.ConfigObj)
 
     if not (modified or force):
         return True
@@ -813,9 +820,11 @@ def save_config(force=False):
                 except KeyError:
                     CFG[sec] = {}
                 value = database[section][option]()
-                if type(value) == type(True):
+                # bool is a subclass of int, check first
+                if isinstance(value, bool):
+                    # convert bool to int when saving so we store 0 or 1
                     CFG[sec][kw] = str(int(value))
-                elif type(value) == type(0):
+                elif isinstance(value, int):
                     CFG[sec][kw] = str(value)
                 else:
                     CFG[sec][kw] = value
@@ -841,6 +850,7 @@ def save_config(force=False):
 
     # Write new config file
     try:
+        logging.info('Writing settings to INI file %s', filename)
         CFG.write()
         shutil.copymode(bakname, filename)
         modified = False
@@ -849,7 +859,7 @@ def save_config(force=False):
         logging.error(T('Cannot write to INI file %s'), filename)
         logging.info("Traceback: ", exc_info=True)
         try:
-            os.remove(filename)
+            sabnzbd.misc.remove_file(filename)
         except:
             pass
         # Restore INI file from backup
@@ -867,13 +877,16 @@ def define_servers():
         for server in CFG['servers']:
             svr = CFG['servers'][server]
             s = ConfigServer(server.replace('{', '[').replace('}', ']'), svr)
-            if s.fillserver():
-                # One time conversion of backup to priority 1
-                s.priority.set(1)
-                s.fillserver.set(False)
+
+            # Conversion of global SSL-Ciphers to server ones
+            if sabnzbd.cfg.ssl_ciphers():
+                s.ssl_ciphers.set(sabnzbd.cfg.ssl_ciphers())
     except KeyError:
         pass
 
+    # No longer needed
+    sabnzbd.cfg.ssl_ciphers.set('')
+
 
 def get_servers():
     global database
@@ -883,7 +896,7 @@ def get_servers():
         return {}
 
 
-def define_categories(force=False):
+def define_categories():
     """ Define categories listed in the Setup file
         return a list of ConfigCat instances
     """
@@ -967,6 +980,25 @@ def define_rss():
 def get_rss():
     global database
     try:
+        # We have to remove non-seperator commas by detecting if they are valid URL's
+        for feed_key in database['rss']:
+            feed = database['rss'][feed_key]
+            # Only modify if we have to, to prevent repeated config-saving
+            have_new_uri = False
+            # Create a new corrected list
+            new_feed_uris = []
+            for feed_uri in feed.uri():
+                if new_feed_uris and not urlparse(feed_uri).scheme and urlparse(new_feed_uris[-1]).scheme:
+                    # Current one has no scheme but previous one does, append to previous
+                    new_feed_uris[-1] += ',' + feed_uri
+                    have_new_uri = True
+                    continue
+                # Add full working URL
+                new_feed_uris.append(feed_uri)
+            # Set new list
+            if have_new_uri:
+                feed.uri.set(new_feed_uris)
+
         return database['rss']
     except KeyError:
         return {}
@@ -1022,6 +1054,14 @@ def no_nonsense(value):
     return None, value
 
 
+def all_lowercase(value):
+    """ Lowercase everything! """
+    if isinstance(value, list):
+        # If list, for each item
+        return None, [item.lower() for item in value]
+    return None, value.lower()
+
+
 def validate_octal(value):
     """ Check if string is valid octal number """
     if not value:
@@ -1054,15 +1094,6 @@ def validate_safedir(root, value, default):
         return T('Error: Queue not empty, cannot change folder.'), None
 
 
-def validate_dir_exists(root, value, default):
-    """ Check if directory exists """
-    p = sabnzbd.misc.real_path(root, value)
-    if os.path.exists(p):
-        return None, value
-    else:
-        return T('Folder "%s" does not exist') % p, None
-
-
 def validate_notempty(root, value, default):
     """ If value is empty, return default """
     if value:
@@ -1071,14 +1102,18 @@ def validate_notempty(root, value, default):
         return None, default
 
 
+def validate_single_tag(value):
+    """ Don't split single indexer tags like "TV > HD"
+        into ['TV', '>', 'HD']
+    """
+    if len(value) == 3:
+        if value[1] == '>':
+            return None, ' '.join(value)
+    return None, value
+
+
 def create_api_key():
     """ Return a new randomized API_KEY """
-    import time
-    try:
-        from hashlib import md5
-    except ImportError:
-        from md5 import md5
-    import random
     # Create some values to seed md5
     t = str(time.time())
     r = str(random.random())
@@ -1089,22 +1124,3 @@ def create_api_key():
 
     # Return a hex digest of the md5, eg 49f68a5c8493ec2c0bf489821c21fc3b
     return m.hexdigest()
-
-
-_FIXES = (
-    ('enable_par_multicore', 'par2_multicore'),
-)
-
-
-def compatibility_fix(cf):
-    """ Convert obsolete INI entries """
-    for item in _FIXES:
-        old, new = item
-        try:
-            cf[new]
-        except KeyError:
-            try:
-                cf[new] = cf[old]
-                del cf[old]
-            except KeyError:
-                pass
