@@ -1,5 +1,5 @@
 #!/usr/bin/python -OO
-# Copyright 2008-2017 The SABnzbd-Team <team@sabnzbd.org>
+# Copyright 2007-2019 The SABnzbd-Team <team@sabnzbd.org>
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -40,6 +40,7 @@ from sabnzbd.constants import DB_HISTORY_NAME, STAGES
 from sabnzbd.encoding import unicoder
 from sabnzbd.bpsmeter import this_week, this_month
 from sabnzbd.decorators import synchronized
+from sabnzbd.misc import int_conv, remove_file, caller_name
 
 DB_LOCK = threading.RLock()
 
@@ -73,8 +74,8 @@ class HistoryDB(object):
     """
     # These class attributes will be accessed directly because
     # they need to be shared by all instances
-    db_path = None        # Will contain full path to history database
-    done_cleaning = False # Ensure we only do one Vacuum per session
+    db_path = None         # Will contain full path to history database
+    done_cleaning = False  # Ensure we only do one Vacuum per session
 
     @synchronized(DB_LOCK)
     def __init__(self):
@@ -84,7 +85,6 @@ class HistoryDB(object):
             HistoryDB.db_path = os.path.join(sabnzbd.cfg.admin_dir.get_path(), DB_HISTORY_NAME)
         self.connect()
 
-
     def connect(self):
         """ Create a connection to the database """
         create_table = not os.path.exists(HistoryDB.db_path)
@@ -111,9 +111,14 @@ class HistoryDB(object):
             _ = self.execute('PRAGMA user_version = 1;') and \
                 self.execute('ALTER TABLE "history" ADD COLUMN series TEXT;') and \
                 self.execute('ALTER TABLE "history" ADD COLUMN md5sum TEXT;')
+        if version < 2:
+            # Add any missing columns added since second DB version
+            # Use "and" to stop when database has been reset due to corruption
+            _ = self.execute('PRAGMA user_version = 2;') and \
+                self.execute('ALTER TABLE "history" ADD COLUMN password TEXT;')
 
     def execute(self, command, args=(), save=False):
-        ''' Wrapper for executing SQL commands '''
+        """ Wrapper for executing SQL commands """
         for tries in xrange(5, 0, -1):
             try:
                 if args and isinstance(args, tuple):
@@ -121,7 +126,7 @@ class HistoryDB(object):
                 else:
                     self.c.execute(command)
                 if save:
-                    self.save()
+                    self.con.commit()
                 return True
             except:
                 error = str(sys.exc_value)
@@ -138,7 +143,7 @@ class HistoryDB(object):
                     logging.info("Traceback: ", exc_info=True)
                     self.close()
                     try:
-                        os.remove(HistoryDB.db_path)
+                        remove_file(HistoryDB.db_path)
                     except:
                         pass
                     self.connect()
@@ -147,7 +152,8 @@ class HistoryDB(object):
                     return 'duplicate column name' not in error
                 else:
                     logging.error(T('SQL Command Failed, see log'))
-                    logging.debug("SQL: %s", command)
+                    logging.info("SQL: %s", command)
+                    logging.info("Arguments: %s", repr(args))
                     logging.info("Traceback: ", exc_info=True)
                     try:
                         self.con.rollback()
@@ -184,18 +190,11 @@ class HistoryDB(object):
             "bytes" INTEGER,
             "meta" TEXT,
             "series" TEXT,
-            "md5sum" TEXT
+            "md5sum" TEXT,
+            "password" TEXT
         )
         """)
-        self.execute('PRAGMA user_version = 1;')
-
-    def save(self):
-        """ Save database to disk """
-        try:
-            self.con.commit()
-        except:
-            logging.error(T('SQL Commit Failed, see log'))
-            logging.info("Traceback: ", exc_info=True)
+        self.execute('PRAGMA user_version = 2;')
 
     def close(self):
         """ Close database connection """
@@ -209,6 +208,7 @@ class HistoryDB(object):
     def remove_completed(self, search=None):
         """ Remove all completed jobs from the database, optional with `search` pattern """
         search = convert_search(search)
+        logging.info('Removing all completed jobs from history')
         return self.execute("""DELETE FROM history WHERE name LIKE ? AND status = 'Completed'""", (search,), save=True)
 
     def get_failed_paths(self, search=None):
@@ -223,6 +223,7 @@ class HistoryDB(object):
     def remove_failed(self, search=None):
         """ Remove all failed jobs from the database, optional with `search` pattern """
         search = convert_search(search)
+        logging.info('Removing all failed jobs from history')
         return self.execute("""DELETE FROM history WHERE name LIKE ? AND status = 'Failed'""", (search,), save=True)
 
     def remove_history(self, jobs=None):
@@ -234,35 +235,58 @@ class HistoryDB(object):
                 jobs = [jobs]
 
             for job in jobs:
-                self.execute("""DELETE FROM history WHERE nzo_id=?""", (job,))
+                self.execute("""DELETE FROM history WHERE nzo_id=?""", (job,), save=True)
+                logging.info('[%s] Removing job %s from history', caller_name(), job)
 
-        self.save()
+    def auto_history_purge(self):
+        """ Remove history items based on the configured history-retention """
+        if sabnzbd.cfg.history_retention() == "0":
+            return
+
+        if sabnzbd.cfg.history_retention() == "-1":
+            # Delete all non-failed ones
+            self.remove_completed()
+
+        if "d" in sabnzbd.cfg.history_retention():
+            # How many days to keep?
+            days_to_keep = int_conv(sabnzbd.cfg.history_retention().strip()[:-1])
+            seconds_to_keep = int(time.time()) - days_to_keep * 86400
+            if days_to_keep > 0:
+                logging.info('Removing completed jobs older than %s days from history', days_to_keep)
+                return self.execute("""DELETE FROM history WHERE status = 'Completed' AND completed < ?""", (seconds_to_keep,), save=True)
+        else:
+            # How many to keep?
+            to_keep = int_conv(sabnzbd.cfg.history_retention())
+            if to_keep > 0:
+                logging.info('Removing all but last %s completed jobs from history', to_keep)
+                return self.execute("""DELETE FROM history WHERE id NOT IN ( SELECT id FROM history WHERE status = 'Completed' ORDER BY completed DESC LIMIT ? )""", (to_keep,), save=True)
 
     def add_history_db(self, nzo, storage, path, postproc_time, script_output, script_line):
         """ Add a new job entry to the database """
         t = build_history_info(nzo, storage, path, postproc_time, script_output, script_line)
 
-        if self.execute("""INSERT INTO history (completed, name, nzb_name, category, pp, script, report,
-        url, status, nzo_id, storage, path, script_log, script_line, download_time, postproc_time, stage_log,
-        downloaded, completeness, fail_message, url_info, bytes, series, md5sum)
-        VALUES (?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?)""", t):
-            self.save()
+        self.execute("""INSERT INTO history (completed, name, nzb_name, category, pp, script, report,
+            url, status, nzo_id, storage, path, script_log, script_line, download_time, postproc_time, stage_log,
+            downloaded, completeness, fail_message, url_info, bytes, series, md5sum, password)
+            VALUES (?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?)""", t, save=True)
+        logging.info('Added job %s to history', nzo.final_name)
 
     def fetch_history(self, start=None, limit=None, search=None, failed_only=0, categories=None):
         """ Return records for specified jobs """
-        search = convert_search(search)
+        command_args = [convert_search(search)]
 
         post = ''
         if categories:
             categories = ['*' if c == 'Default' else c for c in categories]
-            post = " AND (CATEGORY = '"
-            post += "' OR CATEGORY = '".join(categories)
-            post += "' )"
+            post = " AND (CATEGORY = ?"
+            post += " OR CATEGORY = ? " * (len(categories) - 1)
+            post += ")"
+            command_args.extend(categories)
         if failed_only:
             post += ' AND STATUS = "Failed"'
 
         cmd = 'SELECT COUNT(*) FROM history WHERE name LIKE ?'
-        res = self.execute(cmd + post, (search,))
+        res = self.execute(cmd + post, tuple(command_args))
         total_items = -1
         if res:
             try:
@@ -275,9 +299,9 @@ class HistoryDB(object):
         if not limit:
             limit = total_items
 
-        t = (search, start, limit)
+        command_args.extend([start, limit])
         cmd = 'SELECT * FROM history WHERE name LIKE ?'
-        fetch_ok = self.execute(cmd + post + ' ORDER BY completed desc LIMIT ?, ?', t)
+        fetch_ok = self.execute(cmd + post + ' ORDER BY completed desc LIMIT ?, ?', tuple(command_args))
 
         if fetch_ok:
             items = self.c.fetchall()
@@ -290,7 +314,7 @@ class HistoryDB(object):
         # Stage Name is separated by ::: stage lines by ; and stages by \r\n
         items = [unpack_history_info(item) for item in items]
 
-        return (items, fetched_items, total_items)
+        return items, fetched_items, total_items
 
     def have_episode(self, series, season, episode):
         """ Check whether History contains this series episode """
@@ -351,7 +375,7 @@ class HistoryDB(object):
             except AttributeError:
                 pass
 
-        return (total, month, week)
+        return total, month, week
 
     def get_script_log(self, nzo_id):
         """ Return decompressed log file """
@@ -376,7 +400,7 @@ class HistoryDB(object):
         return name
 
     def get_path(self, nzo_id):
-        """ Return the `incomplete` path of the job `nzo_id` """
+        """ Return the `incomplete` path of the job `nzo_id` if it is still there """
         t = (nzo_id,)
         path = ''
         if self.execute('SELECT path FROM history WHERE nzo_id=?', t):
@@ -384,7 +408,9 @@ class HistoryDB(object):
                 path = self.c.fetchone().get('path')
             except AttributeError:
                 pass
-        return path
+        if os.path.exists(path):
+            return path
+        return None
 
     def get_other(self, nzo_id):
         """ Return additional data for job `nzo_id` """
@@ -397,9 +423,10 @@ class HistoryDB(object):
                 pp = items.get('pp')
                 script = items.get('script')
                 cat = items.get('category')
+                return dtype, url, pp, script, cat
             except (AttributeError, IndexError):
-                return '', '', '', '', ''
-        return dtype, url, pp, script, cat
+                pass
+        return '', '', '', '', ''
 
 
 def dict_factory(cursor, row):
@@ -471,8 +498,7 @@ def build_history_info(nzo, storage='', downpath='', postproc_time=0, script_out
 
     return (completed, name, nzb_name, category, pp, script, report, url, status, nzo_id, storage, path,
             script_log, script_line, download_time, postproc_time, stage_log, downloaded, completeness,
-            fail_message, url_info, bytes, series, nzo.md5sum)
-
+            fail_message, url_info, bytes, series, nzo.md5sum, nzo.password)
 
 
 def unpack_history_info(item):
@@ -517,11 +543,18 @@ def unpack_history_info(item):
     if item['script_log']:
         item['script_log'] = ''
     # The action line is only available for items in the postproc queue
-    if not item.has_key('action_line'):
+    if 'action_line' not in item:
         item['action_line'] = ''
     return item
 
 
+def midnight_history_purge():
+    logging.info('Scheduled history purge')
+    history_db = HistoryDB()
+    history_db.auto_history_purge()
+    history_db.close()
+
+
 def decode_factory(text):
     """ Recursively looks through the supplied argument
         and converts and text to Unicode
