@@ -1,5 +1,5 @@
 #!/usr/bin/python -OO
-# Copyright 2008-2017 The SABnzbd-Team <team@sabnzbd.org>
+# Copyright 2007-2019 The SABnzbd-Team <team@sabnzbd.org>
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -20,29 +20,33 @@
 ##############################################################################
 from threading import RLock, Condition
 
+
+# All operations that modify the queue need to happen in a lock
+# Also used when importing NZBs to prevent IO-race conditions
+# Names of wrapper-functions should be the same in misc.caller_name
+# The NzbQueueLocker both locks and notifies the Downloader
 NZBQUEUE_LOCK = RLock()
-CV = Condition(NZBQUEUE_LOCK)
-IO_LOCK = RLock()
+DOWNLOADER_CV = Condition(NZBQUEUE_LOCK)
 
 def synchronized(lock):
     def wrap(f):
-        def newFunction(*args, **kw):
+        def call_func(*args, **kw):
             lock.acquire()
             try:
                 return f(*args, **kw)
             finally:
                 lock.release()
-        return newFunction
+        return call_func
     return wrap
 
 
-def synchronized_CV(func):
-    global CV
+def NzbQueueLocker(func):
+    global DOWNLOADER_CV
     def call_func(*params, **kparams):
-        CV.acquire()
+        DOWNLOADER_CV.acquire()
         try:
             return func(*params, **kparams)
         finally:
-            CV.notifyAll()
-            CV.release()
+            DOWNLOADER_CV.notify_all()
+            DOWNLOADER_CV.release()
     return call_func
