@@ -1,5 +1,5 @@
 #!/usr/bin/python -OO
-# Copyright 2008-2017 The SABnzbd-Team <team@sabnzbd.org>
+# Copyright 2007-2019 The SABnzbd-Team <team@sabnzbd.org>
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -27,9 +27,11 @@ from nntplib import NNTPPermanentError
 import socket
 import random
 import sys
+import Queue
 
 import sabnzbd
-from sabnzbd.decorators import synchronized, synchronized_CV, CV
+from sabnzbd.decorators import synchronized, NzbQueueLocker, DOWNLOADER_CV
+from sabnzbd.constants import MAX_DECODE_QUEUE, LIMIT_DECODE_QUEUE
 from sabnzbd.decoder import Decoder
 from sabnzbd.newswrapper import NewsWrapper, request_server_info
 from sabnzbd.articlecache import ArticleCache
@@ -38,7 +40,7 @@ import sabnzbd.config as config
 import sabnzbd.cfg as cfg
 from sabnzbd.bpsmeter import BPSMeter
 import sabnzbd.scheduler
-from sabnzbd.misc import from_units
+from sabnzbd.misc import from_units, nntp_to_msg
 from sabnzbd.utils.happyeyeballs import happyeyeballs
 
 
@@ -58,8 +60,8 @@ TIMER_LOCK = RLock()
 
 class Server(object):
 
-    def __init__(self, id, displayname, host, port, timeout, threads, priority, ssl, ssl_verify, send_group, username=None,
-                 password=None, optional=False, retention=0, categories=None):
+    def __init__(self, id, displayname, host, port, timeout, threads, priority, ssl, ssl_verify, ssl_ciphers,
+                 send_group, username=None, password=None, optional=False, retention=0):
 
         self.id = id
         self.newid = None
@@ -72,6 +74,7 @@ class Server(object):
         self.priority = priority
         self.ssl = ssl
         self.ssl_verify = ssl_verify
+        self.ssl_ciphers = ssl_ciphers
         self.optional = optional
         self.retention = retention
         self.send_group = send_group
@@ -79,8 +82,6 @@ class Server(object):
         self.username = username
         self.password = password
 
-        self.categories = categories
-
         self.busy_threads = []
         self.idle_threads = []
         self.active = True
@@ -88,7 +89,7 @@ class Server(object):
         self.errormsg = ''
         self.warning = ''
         self.info = None     # Will hold getaddrinfo() list
-        self.ssl_info = '' # Will hold the type and cipher of SSL connection
+        self.ssl_info = ''  # Will hold the type and cipher of SSL connection
         self.request = False  # True if a getaddrinfo() request is pending
         self.have_body = 'free.xsusenet.com' not in host
         self.have_stat = True  # Assume server has "STAT", until proven otherwise
@@ -105,7 +106,7 @@ class Server(object):
             2 - and self.info has more than 1 entry (read: IP address): Return the quickest IP based on the happyeyeballs algorithm
             In case of problems: return the host name itself
         """
-        # Check if already a succesfull ongoing connection
+        # Check if already a successful ongoing connection
         if self.busy_threads and self.busy_threads[0].nntp:
             # Re-use that IP
             logging.debug('%s: Re-using address %s', self.host, self.busy_threads[0].nntp.host)
@@ -191,12 +192,21 @@ class Downloader(Thread):
         self.write_fds = {}
 
         self.servers = []
+        self.server_dict = {} # For faster lookups, but is not updated later!
+        self.server_nr = 0
         self._timers = {}
 
         for server in config.get_servers():
             self.init_server(None, server)
 
-        self.decoder = Decoder(self.servers)
+        self.decoder_queue = Queue.Queue()
+
+        # Initialize decoders, only 1 for non-SABYenc
+        self.decoder_workers = []
+        nr_decoders = 2 if sabnzbd.decoder.SABYENC_ENABLED else 1
+        for i in range(nr_decoders):
+            self.decoder_workers.append(Decoder(self.servers, self.decoder_queue))
+
         Downloader.do = self
 
     def init_server(self, oldserver, newserver):
@@ -217,12 +227,12 @@ class Downloader(Thread):
             timeout = srv.timeout()
             threads = srv.connections()
             priority = srv.priority()
-            ssl = srv.ssl() and sabnzbd.HAVE_SSL
+            ssl = srv.ssl()
             ssl_verify = srv.ssl_verify()
+            ssl_ciphers = srv.ssl_ciphers()
             username = srv.username()
             password = srv.password()
             optional = srv.optional()
-            categories = srv.categories()
             retention = float(srv.retention() * 24 * 3600)  # days ==> seconds
             send_group = srv.send_group()
             create = True
@@ -238,17 +248,22 @@ class Downloader(Thread):
                     break
 
         if create and enabled and host and port and threads:
-            self.servers.append(Server(newserver, displayname, host, port, timeout, threads, priority, ssl, ssl_verify,
-                                            send_group, username, password, optional, retention, categories=categories))
+            server = Server(newserver, displayname, host, port, timeout, threads, priority, ssl, ssl_verify,
+                                    ssl_ciphers, send_group, username, password, optional, retention)
+            self.servers.append(server)
+            self.server_dict[newserver] = server
+
+        # Update server-count
+        self.server_nr = len(self.servers)
 
         return
 
-    @synchronized_CV
+    @NzbQueueLocker
     def set_paused_state(self, state):
         """ Set downloader to specified paused state """
         self.paused = state
 
-    @synchronized_CV
+    @NzbQueueLocker
     def resume(self):
         # Do not notify when SABnzbd is still starting
         if self.paused and sabnzbd.WEB_DIR:
@@ -256,37 +271,32 @@ class Downloader(Thread):
             notifier.send_notification("SABnzbd", T('Resuming'), 'download')
         self.paused = False
 
-    @synchronized_CV
-    def pause(self, save=True):
+    @NzbQueueLocker
+    def pause(self):
         """ Pause the downloader, optionally saving admin """
         if not self.paused:
             self.paused = True
-            self.can_be_slowed = None
             logging.info("Pausing")
             notifier.send_notification("SABnzbd", T('Paused'), 'download')
             if self.is_paused():
                 BPSMeter.do.reset()
             if cfg.autodisconnect():
                 self.disconnect()
-            if save:
-                ArticleCache.do.flush_articles()
 
-    @synchronized_CV
     def delay(self):
         logging.debug("Delaying")
         self.delayed = True
 
-    @synchronized_CV
+    @NzbQueueLocker
     def undelay(self):
         logging.debug("Undelaying")
         self.delayed = False
 
-    @synchronized_CV
     def wait_for_postproc(self):
         logging.info("Waiting for post-processing to finish")
         self.postproc = True
 
-    @synchronized_CV
+    @NzbQueueLocker
     def resume_from_postproc(self):
         logging.info("Post-processing finished, resuming download")
         self.postproc = False
@@ -294,15 +304,14 @@ class Downloader(Thread):
     def disconnect(self):
         self.force_disconnect = True
 
-    @synchronized_CV
     def limit_speed(self, value):
-        ''' Set the actual download speed in Bytes/sec
+        """ Set the actual download speed in Bytes/sec
             When 'value' ends with a '%' sign or is within 1-100, it is interpreted as a pecentage of the maximum bandwidth
             When no '%' is found, it is interpreted as an absolute speed (including KMGT notation).
-        '''
+        """
         if value:
             mx = cfg.bandwidth_max.get_int()
-            if '%' in str(value) or (from_units(value) > 0 and from_units(value) < 101):
+            if '%' in str(value) or (0 < from_units(value) < 101):
                 limit = value.strip(' %')
                 self.bandwidth_perc = from_units(limit)
                 if mx:
@@ -318,7 +327,6 @@ class Downloader(Thread):
         else:
             self.speed_set()
         logging.info("Speed limit set to %s B/s", self.bandwidth_limit)
-        self.can_be_slowed = None
 
     def get_limit(self):
         return self.bandwidth_perc
@@ -349,7 +357,6 @@ class Downloader(Thread):
         """ Return True when this server has the highest priority of the active ones
             0 is the highest priority
         """
-
         for server in self.servers:
             if server is not me and server.active and server.priority < me.priority:
                 return False
@@ -359,14 +366,27 @@ class Downloader(Thread):
         return filter(nzo.server_in_try_list, self.servers)
 
     def maybe_block_server(self, server):
-        if server.optional and server.active and (server.bad_cons / server.threads) > 3:
-            # Optional and active server had too many problems,
-            # disable it now and send a re-enable plan to the scheduler
+        # Was it resolving problem?
+        if server.info is False:
+            # Warn about resolving issues
+            errormsg = T('Cannot connect to server %s [%s]') % (server.host, T('Server name does not resolve'))
+            if server.errormsg != errormsg:
+                server.errormsg = errormsg
+                logging.warning(errormsg)
+                logging.warning(T('Server %s will be ignored for %s minutes'), server.host, _PENALTY_TIMEOUT)
+
+            # Not fully the same as the code below for optional servers
             server.bad_cons = 0
             server.active = False
-            server.errormsg = T('Server %s will be ignored for %s minutes') % ('', _PENALTY_TIMEOUT)
-            logging.warning(T('Server %s will be ignored for %s minutes'), server.id, _PENALTY_TIMEOUT)
-            self.plan_server(server.id, _PENALTY_TIMEOUT)
+            self.plan_server(server, _PENALTY_TIMEOUT)
+
+        # Optional and active server had too many problems.
+        # Disable it now and send a re-enable plan to the scheduler
+        if server.optional and server.active and (server.bad_cons / server.threads) > 3:
+            server.bad_cons = 0
+            server.active = False
+            logging.warning(T('Server %s will be ignored for %s minutes'), server.host, _PENALTY_TIMEOUT)
+            self.plan_server(server, _PENALTY_TIMEOUT)
 
             # Remove all connections to server
             for nw in server.idle_threads + server.busy_threads:
@@ -376,36 +396,32 @@ class Downloader(Thread):
 
             sabnzbd.nzbqueue.NzbQueue.do.reset_all_try_lists()
 
+    def decode(self, article, lines, raw_data):
+        self.decoder_queue.put((article, lines, raw_data))
+        # See if there's space left in cache, pause otherwise
+        # But do allow some articles to enter queue, in case of full cache
+        qsize = self.decoder_queue.qsize()
+        if (not ArticleCache.do.reserve_space(lines) and qsize > MAX_DECODE_QUEUE) or (qsize > LIMIT_DECODE_QUEUE):
+            sabnzbd.downloader.Downloader.do.delay()
+
     def run(self):
         # First check IPv6 connectivity
         sabnzbd.EXTERNAL_IPV6 = sabnzbd.test_ipv6()
         logging.debug('External IPv6 test result: %s', sabnzbd.EXTERNAL_IPV6)
 
-        # Then have to check the quality of SSL verification
-        if sabnzbd.HAVE_SSL_CONTEXT:
-            try:
-                import ssl
-                ctx = ssl.create_default_context()
-                base_sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
-                ssl_sock = ctx.wrap_socket(base_sock, server_hostname=cfg.selftest_host())
-                ssl_sock.settimeout(2.0)
-                ssl_sock.connect((cfg.selftest_host(), 443))
-                ssl_sock.close()
-            except:
-                # Seems something is still wrong
-                sabnzbd.set_https_verification(0)
-                sabnzbd.HAVE_SSL_CONTEXT = False
+        # Then we check SSL certifcate checking
+        sabnzbd.HAVE_SSL_CONTEXT = sabnzbd.test_cert_checking()
         logging.debug('SSL verification test: %s', sabnzbd.HAVE_SSL_CONTEXT)
 
-        # Start decoder
-        self.decoder.start()
+        # Start decoders
+        for decoder in self.decoder_workers:
+            decoder.start()
 
         # Kick BPS-Meter to check quota
         BPSMeter.do.update()
 
         while 1:
             for server in self.servers:
-                if 0: assert isinstance(server, Server) # Assert only for debug purposes
                 for nw in server.busy_threads[:]:
                     if (nw.nntp and nw.nntp.error_msg) or (nw.timeout and time.time() > nw.timeout):
                         if nw.nntp and nw.nntp.error_msg:
@@ -429,7 +445,6 @@ class Downloader(Thread):
                         # Restart pending, don't add new articles
                         continue
 
-                if 0: assert isinstance(server, Server) # Assert only for debug purposes
                 if not server.idle_threads or server.restart or self.is_paused() or self.shutdown or self.delayed or self.postproc:
                     continue
 
@@ -437,16 +452,17 @@ class Downloader(Thread):
                     continue
 
                 for nw in server.idle_threads[:]:
-                    if 0: assert isinstance(nw, NewsWrapper) # Assert only for debug purposes
                     if nw.timeout:
                         if time.time() < nw.timeout:
                             continue
                         else:
                             nw.timeout = None
 
-                    if server.info is None:
-                        self.maybe_block_server(server)
-                        request_server_info(server)
+                    if not server.info:
+                        # Only request info if there's stuff in the queue
+                        if not sabnzbd.nzbqueue.NzbQueue.do.is_empty():
+                            self.maybe_block_server(server)
+                            request_server_info(server)
                         break
 
                     article = sabnzbd.nzbqueue.NzbQueue.do.get_article(server, self.servers)
@@ -455,10 +471,11 @@ class Downloader(Thread):
                         break
 
                     if server.retention and article.nzf.nzo.avg_stamp < time.time() - server.retention:
-                        # Article too old for the server, treat as missing
-                        if sabnzbd.LOG_ALL:
-                            logging.debug('Article %s too old for %s', article.article, server.id)
-                        self.decoder.decode(article, None)
+                        # Let's get rid of all the articles for this server at once
+                        logging.info('Job %s too old for %s, moving on', article.nzf.nzo.work_name, server.host)
+                        while article:
+                            self.decode(article, None, None)
+                            article = article.nzf.nzo.get_article(server, self.servers)
                         break
 
                     server.idle_threads.remove(nw)
@@ -470,11 +487,10 @@ class Downloader(Thread):
                         self.__request_article(nw)
                     else:
                         try:
-                            logging.info("%s@%s: Initiating connection",
-                                              nw.thrdnum, server.id)
+                            logging.info("%s@%s: Initiating connection", nw.thrdnum, server.host)
                             nw.init_connect(self.write_fds)
                         except:
-                            logging.error(T('Failed to initialize %s@%s with reason: %s'), nw.thrdnum, server.id, sys.exc_info()[1])
+                            logging.error(T('Failed to initialize %s@%s with reason: %s'), nw.thrdnum, server.host, sys.exc_info()[1])
                             self.__reset_nw(nw, "failed to initialize")
 
             # Exit-point
@@ -486,8 +502,10 @@ class Downloader(Thread):
                         break
 
                 if empty:
-                    self.decoder.stop()
-                    self.decoder.join()
+                    # Start decoders
+                    for decoder in self.decoder_workers:
+                        decoder.stop()
+                        decoder.join()
 
                     for server in self.servers:
                         server.stop(self.read_fds, self.write_fds)
@@ -512,20 +530,20 @@ class Downloader(Thread):
             if readkeys or writekeys:
                 read, write, error = select.select(readkeys, writekeys, (), 1.0)
 
-                # Why check so often when so few things happend?
+                # Why check so often when so few things happened?
                 if self.can_be_slowed and len(readkeys) >= 8 and len(read) <= 2:
                     time.sleep(0.01)
 
-                # Need to initalize the check during first 20 seconds
+                # Need to initialize the check during first 20 seconds
                 if self.can_be_slowed is None or self.can_be_slowed_timer:
                     # Wait for stable speed to start testing
                     if not self.can_be_slowed_timer and BPSMeter.do.get_stable_speed(timespan=10):
                         self.can_be_slowed_timer = time.time()
 
-                    # Check 10 seconds after enabeling slowdown
+                    # Check 10 seconds after enabling slowdown
                     if self.can_be_slowed_timer and time.time() > self.can_be_slowed_timer + 10:
                         # Now let's check if it was stable in the last 10 seconds
-                        self.can_be_slowed = (BPSMeter.do.get_stable_speed(timespan=10) > 0)
+                        self.can_be_slowed = BPSMeter.do.get_stable_speed(timespan=10)
                         self.can_be_slowed_timer = 0
                         logging.debug('Downloader-slowdown: %r', self.can_be_slowed)
 
@@ -536,11 +554,11 @@ class Downloader(Thread):
 
                 time.sleep(1.0)
 
-                CV.acquire()
+                DOWNLOADER_CV.acquire()
                 while (sabnzbd.nzbqueue.NzbQueue.do.is_empty() or self.is_paused() or self.delayed or self.postproc) and not \
                        self.shutdown and not self.__restart:
-                    CV.wait()
-                CV.release()
+                    DOWNLOADER_CV.wait()
+                DOWNLOADER_CV.release()
 
                 self.force_disconnect = False
 
@@ -594,17 +612,15 @@ class Downloader(Thread):
                     if nzo:
                         nzo.update_download_stats(BPSMeter.do.get_bps(), server.id, bytes)
 
-                if len(nw.lines) == 1:
-                    code = nw.lines[0][:3]
-                    if not nw.connected or code == '480':
+                if not done and nw.status_code != '222':
+                    if not nw.connected or nw.status_code == '480':
                         done = False
 
                         try:
-                            nw.finish_connect(code)
+                            nw.finish_connect(nw.status_code)
                             if sabnzbd.LOG_ALL:
-                                logging.debug("%s@%s last message -> %s", nw.thrdnum, nw.server.id, nw.lines[0])
-                            nw.lines = []
-                            nw.data = ''
+                                logging.debug("%s@%s last message -> %s", nw.thrdnum, nw.server.host, nntp_to_msg(nw.data))
+                            nw.clear_data()
                         except NNTPPermanentError, error:
                             # Handle login problems
                             block = False
@@ -620,9 +636,9 @@ class Downloader(Thread):
                                     errormsg = T('Too many connections to server %s') % display_msg
                                     if server.errormsg != errormsg:
                                         server.errormsg = errormsg
-                                        logging.warning(T('Too many connections to server %s'), server.id)
+                                        logging.warning(T('Too many connections to server %s'), server.host)
                                     self.__reset_nw(nw, None, warn=False, destroy=True, quit=True)
-                                    self.plan_server(server.id, _PENALTY_TOOMANY)
+                                    self.plan_server(server, _PENALTY_TOOMANY)
                                     server.threads -= 1
                             elif ecode in ('502', '481', '482') and clues_too_many_ip(msg):
                                 # Account sharing?
@@ -630,25 +646,26 @@ class Downloader(Thread):
                                     errormsg = T('Probable account sharing') + display_msg
                                     if server.errormsg != errormsg:
                                         server.errormsg = errormsg
-                                        name = ' (%s)' % server.id
+                                        name = ' (%s)' % server.host
                                         logging.warning(T('Probable account sharing') + name)
-                                    penalty = _PENALTY_SHARE
+                                penalty = _PENALTY_SHARE
+                                block = True
                             elif ecode in ('481', '482', '381') or (ecode == '502' and clues_login(msg)):
                                 # Cannot login, block this server
                                 if server.active:
                                     errormsg = T('Failed login for server %s') % display_msg
                                     if server.errormsg != errormsg:
                                         server.errormsg = errormsg
-                                        logging.error(T('Failed login for server %s'), server.id)
+                                        logging.error(T('Failed login for server %s'), server.host)
                                 penalty = _PENALTY_PERM
                                 block = True
-                            elif ecode == '502':
+                            elif ecode in ('502', '482'):
                                 # Cannot connect (other reasons), block this server
                                 if server.active:
                                     errormsg = T('Cannot connect to server %s [%s]') % ('', display_msg)
                                     if server.errormsg != errormsg:
                                         server.errormsg = errormsg
-                                        logging.warning(T('Cannot connect to server %s [%s]'), server.id, msg)
+                                        logging.warning(T('Cannot connect to server %s [%s]'), server.host, msg)
                                 if clues_pay(msg):
                                     penalty = _PENALTY_PERM
                                 else:
@@ -657,7 +674,7 @@ class Downloader(Thread):
                             elif ecode == '400':
                                 # Temp connection problem?
                                 if server.active:
-                                    logging.debug('Unspecified error 400 from server %s', server.id)
+                                    logging.debug('Unspecified error 400 from server %s', server.host)
                                 penalty = _PENALTY_VERYSHORT
                                 block = True
                             else:
@@ -666,77 +683,71 @@ class Downloader(Thread):
                                     errormsg = T('Cannot connect to server %s [%s]') % ('', display_msg)
                                     if server.errormsg != errormsg:
                                         server.errormsg = errormsg
-                                        logging.warning(T('Cannot connect to server %s [%s]'), server.id, msg)
-                                    penalty = _PENALTY_UNKNOWN
+                                        logging.warning(T('Cannot connect to server %s [%s]'), server.host, msg)
+                                penalty = _PENALTY_UNKNOWN
+                                block = True
                             if block or (penalty and server.optional):
                                 if server.active:
                                     server.active = False
                                     if penalty and (block or server.optional):
-                                        self.plan_server(server.id, penalty)
+                                        self.plan_server(server, penalty)
                                     sabnzbd.nzbqueue.NzbQueue.do.reset_all_try_lists()
                                 self.__reset_nw(nw, None, warn=False, quit=True)
                             continue
                         except:
                             logging.error(T('Connecting %s@%s failed, message=%s'),
-                                              nw.thrdnum, nw.server.id, nw.lines[0])
+                                              nw.thrdnum, nw.server.host, nntp_to_msg(nw.data))
                             # No reset-warning needed, above logging is sufficient
                             self.__reset_nw(nw, None, warn=False)
 
                         if nw.connected:
-                            logging.info("Connecting %s@%s finished", nw.thrdnum, nw.server.id)
+                            logging.info("Connecting %s@%s finished", nw.thrdnum, nw.server.host)
                             self.__request_article(nw)
 
-                    elif code == '223':
+                    elif nw.status_code == '223':
                         done = True
                         logging.debug('Article <%s> is present', article.article)
-                        self.decoder.decode(article, nw.lines)
 
-                    elif code == '211':
+                    elif nw.status_code == '211':
                         done = False
-
-                        logging.debug("group command ok -> %s",
-                                      nw.lines)
+                        logging.debug("group command ok -> %s", nntp_to_msg(nw.data))
                         nw.group = nw.article.nzf.nzo.group
-                        nw.lines = []
-                        nw.data = ''
+                        nw.clear_data()
                         self.__request_article(nw)
 
-                    elif code in ('411', '423', '430'):
+                    elif nw.status_code in ('411', '423', '430'):
                         done = True
-                        nw.lines = None
+                        logging.debug('Thread %s@%s: Article %s missing (error=%s)',
+                                        nw.thrdnum, nw.server.host, article.article, nw.status_code)
+                        nw.clear_data()
 
-                        logging.info('Thread %s@%s: Article ' +
-                                        '%s missing (error=%s)',
-                                        nw.thrdnum, nw.server.id, article.article, code)
-
-                    elif code == '480':
+                    elif nw.status_code == '480':
                         if server.active:
                             server.active = False
                             server.errormsg = T('Server %s requires user/password') % ''
-                            self.plan_server(server.id, 0)
+                            self.plan_server(server, 0)
                             sabnzbd.nzbqueue.NzbQueue.do.reset_all_try_lists()
-                        msg = T('Server %s requires user/password') % nw.server.id
+                        msg = T('Server %s requires user/password') % nw.server.host
                         self.__reset_nw(nw, msg, quit=True)
 
-                    elif code == '500':
+                    elif nw.status_code == '500':
                         if nzo.precheck:
                             # Assume "STAT" command is not supported
                             server.have_stat = False
-                            logging.debug('Server %s does not support STAT', server.id)
+                            logging.debug('Server %s does not support STAT', server.host)
                         else:
                             # Assume "BODY" command is not supported
                             server.have_body = False
-                            logging.debug('Server %s does not support BODY', server.id)
-                        nw.lines = []
-                        nw.data = ''
+                            logging.debug('Server %s does not support BODY', server.host)
+                        nw.clear_data()
                         self.__request_article(nw)
 
                 if done:
                     server.bad_cons = 0  # Succesful data, clear "bad" counter
                     server.errormsg = server.warning = ''
                     if sabnzbd.LOG_ALL:
-                        logging.debug('Thread %s@%s: %s done', nw.thrdnum, server.id, article.article)
-                    self.decoder.decode(article, nw.lines)
+                        logging.debug('Thread %s@%s: %s done', nw.thrdnum, server.host, article.article)
+                    self.decode(article, nw.lines, nw.data)
 
                     nw.soft_reset()
                     server.busy_threads.remove(nw)
@@ -767,9 +778,9 @@ class Downloader(Thread):
 
         if warn and errormsg:
             server.warning = errormsg
-            logging.info('Thread %s@%s: ' + errormsg, nw.thrdnum, server.id)
+            logging.info('Thread %s@%s: ' + errormsg, nw.thrdnum, server.host)
         elif errormsg:
-            logging.info('Thread %s@%s: ' + errormsg, nw.thrdnum, server.id)
+            logging.info('Thread %s@%s: ' + errormsg, nw.thrdnum, server.host)
 
         if nw in server.busy_threads:
             server.busy_threads.remove(nw)
@@ -784,16 +795,10 @@ class Downloader(Thread):
         if article:
             if article.tries > cfg.max_art_tries() and (article.fetcher.optional or not cfg.max_art_opt()):
                 # Too many tries on this server, consider article missing
-                self.decoder.decode(article, None)
+                self.decode(article, None, None)
             else:
-                # Remove this server from try_list
-                article.fetcher = None
-
-                nzf = article.nzf
-                nzo = nzf.nzo
-
-                # Allow all servers to iterate over each nzo/nzf again ##
-                sabnzbd.nzbqueue.NzbQueue.do.reset_try_lists(nzf, nzo)
+                # Allow all servers to iterate over each nzo/nzf again
+                sabnzbd.nzbqueue.NzbQueue.do.reset_try_lists(article)
 
         if destroy:
             nw.terminate(quit=quit)
@@ -809,11 +814,11 @@ class Downloader(Thread):
             if nw.server.send_group and nzo.group != nw.group:
                 group = nzo.group
                 if sabnzbd.LOG_ALL:
-                    logging.debug('Thread %s@%s: GROUP <%s>', nw.thrdnum, nw.server.id, group)
+                    logging.debug('Thread %s@%s: GROUP <%s>', nw.thrdnum, nw.server.host, group)
                 nw.send_group(group)
             else:
                 if sabnzbd.LOG_ALL:
-                    logging.debug('Thread %s@%s: BODY %s', nw.thrdnum, nw.server.id, nw.article.article)
+                    logging.debug('Thread %s@%s: BODY %s', nw.thrdnum, nw.server.host, nw.article.article)
                 nw.body(nzo.precheck)
 
             fileno = nw.nntp.sock.fileno()
@@ -835,30 +840,30 @@ class Downloader(Thread):
     # Each server has a dictionary entry, consisting of a list of timestamps.
 
     @synchronized(TIMER_LOCK)
-    def plan_server(self, server_id, interval):
+    def plan_server(self, server, interval):
         """ Plan the restart of a server in 'interval' minutes """
         if cfg.no_penalties() and interval > _PENALTY_SHORT:
             # Overwrite in case of no_penalties
             interval = _PENALTY_SHORT
 
-        logging.debug('Set planned server resume %s in %s mins', server_id, interval)
-        if server_id not in self._timers:
-            self._timers[server_id] = []
+        logging.debug('Set planned server resume %s in %s mins', server.host, interval)
+        if server.id not in self._timers:
+            self._timers[server.id] = []
         stamp = time.time() + 60.0 * interval
-        self._timers[server_id].append(stamp)
+        self._timers[server.id].append(stamp)
         if interval:
-            sabnzbd.scheduler.plan_server(self.trigger_server, [server_id, stamp], interval)
+            sabnzbd.scheduler.plan_server(self.trigger_server, [server.id, stamp], interval)
 
     @synchronized(TIMER_LOCK)
     def trigger_server(self, server_id, timestamp):
         """ Called by scheduler, start server if timer still valid """
-        logging.debug('Trigger planned server resume %s', server_id)
+        logging.debug('Trigger planned server resume for server-id %s', server_id)
         if server_id in self._timers:
             if timestamp in self._timers[server_id]:
                 del self._timers[server_id]
                 self.init_server(server_id, server_id)
 
-    @synchronized_CV
+    @NzbQueueLocker
     @synchronized(TIMER_LOCK)
     def unblock(self, server_id):
         # Remove timer
@@ -869,7 +874,7 @@ class Downloader(Thread):
         # Activate server if it was inactive
         for server in self.servers:
             if server.id == server_id and not server.active:
-                logging.debug('Unblock server %s', server_id)
+                logging.debug('Unblock server %s', server.host)
                 self.init_server(server_id, server_id)
                 break
 
@@ -877,7 +882,7 @@ class Downloader(Thread):
         for server_id in self._timers.keys():
             self.unblock(server_id)
 
-    @synchronized_CV
+    @NzbQueueLocker
     @synchronized(TIMER_LOCK)
     def check_timers(self):
         """ Make sure every server without a non-expired timer is active """
@@ -886,7 +891,7 @@ class Downloader(Thread):
         kicked = []
         for server_id in self._timers.keys():
             if not [stamp for stamp in self._timers[server_id] if stamp >= now]:
-                logging.debug('Forcing re-evaluation of server %s', server_id)
+                logging.debug('Forcing re-evaluation of server-id %s', server_id)
                 del self._timers[server_id]
                 self.init_server(server_id, server_id)
                 kicked.append(server_id)
@@ -894,14 +899,16 @@ class Downloader(Thread):
         for server in self.servers:
             if server.id not in self._timers:
                 if server.id not in kicked and not server.active:
-                    logging.debug('Forcing activation of server %s', server.id)
+                    logging.debug('Forcing activation of server %s', server.host)
                     self.init_server(server.id, server.id)
 
-    @synchronized_CV
     def update_server(self, oldserver, newserver):
+        """ Update the server and make sure we trigger
+            the update in the loop to do housekeeping """
         self.init_server(oldserver, newserver)
+        self.wakeup()
 
-    @synchronized_CV
+    @NzbQueueLocker
     def wakeup(self):
         """ Just rattle the semaphore """
         pass
@@ -912,12 +919,12 @@ class Downloader(Thread):
 
 
 def stop():
-    CV.acquire()
+    DOWNLOADER_CV.acquire()
     try:
         Downloader.do.stop()
     finally:
-        CV.notifyAll()
-        CV.release()
+        DOWNLOADER_CV.notify_all()
+        DOWNLOADER_CV.release()
     try:
         Downloader.do.join()
     except:
@@ -937,7 +944,8 @@ def clues_too_many(text):
     """ Check for any "too many connections" clues in the response code """
     text = text.lower()
     for clue in ('exceed', 'connections', 'too many', 'threads', 'limit'):
-        if clue in text:
+        # Not 'download limit exceeded' error
+        if (clue in text) and ('download' not in text) and ('byte' not in text):
             return True
     return False
 
@@ -954,7 +962,7 @@ def clues_too_many_ip(text):
 def clues_pay(text):
     """ Check for messages about payments """
     text = text.lower()
-    for clue in ('credits', 'paym', 'expired'):
+    for clue in ('credits', 'paym', 'expired', 'exceeded'):
         if clue in text:
             return True
     return False
