@@ -1,5 +1,5 @@
 #!/usr/bin/python -OO
-# Copyright 2008-2017 The SABnzbd-Team <team@sabnzbd.org>
+# Copyright 2007-2019 The SABnzbd-Team <team@sabnzbd.org>
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -24,6 +24,7 @@ import sys
 import logging
 import urllib
 import re
+import ctypes
 import shutil
 import threading
 import subprocess
@@ -32,19 +33,24 @@ import time
 import datetime
 import fnmatch
 import stat
+import inspect
+import urllib2
 from urlparse import urlparse
 
 import sabnzbd
 from sabnzbd.decorators import synchronized
 from sabnzbd.constants import DEFAULT_PRIORITY, FUTURE_Q_FOLDER, JOB_ADMIN, \
-     GIGI, MEBI, DEF_CACHE_LIMIT
+     GIGI, MEBI, DEF_ARTICLE_CACHE_DEFAULT, DEF_ARTICLE_CACHE_MAX
 import sabnzbd.config as config
 import sabnzbd.cfg as cfg
 from sabnzbd.encoding import unicoder, special_fixer, gUTF
+import sabnzbd.utils.rarfile as rarfile
 
-RE_VERSION = re.compile(r'(\d+)\.(\d+)\.(\d+)([a-zA-Z]*)(\d*)')
-RE_UNITS = re.compile(r'(\d+\.*\d*)\s*([KMGTP]{0,1})', re.I)
 TAB_UNITS = ('', 'K', 'M', 'G', 'T', 'P')
+RE_UNITS = re.compile(r'(\d+\.*\d*)\s*([KMGTP]{0,1})', re.I)
+RE_VERSION = re.compile(r'(\d+)\.(\d+)\.(\d+)([a-zA-Z]*)(\d*)')
+RE_IP4 = re.compile(r'inet\s+(addr:\s*){0,1}(\d+\.\d+\.\d+\.\d+)')
+RE_IP6 = re.compile(r'inet6\s+(addr:\s*){0,1}([0-9a-f:]+)', re.I)
 
 # Check if strings are defined for AM and PM
 HAVE_AMPM = bool(time.strftime('%p', time.localtime()))
@@ -92,6 +98,15 @@ def calc_age(date, trans=False):
     return age
 
 
+def monthrange(start, finish):
+    """ Calculate months between 2 dates, used in the Config template """
+    months = (finish.year - start.year) * 12 + finish.month + 1
+    for i in xrange(start.month, months):
+        year  = (i - 1) / 12 + start.year
+        month = (i - 1) % 12 + 1
+        yield datetime.date(year, month, 1)
+
+
 def safe_lower(txt):
     """ Return lowercased string. Return '' for None """
     if txt:
@@ -100,13 +115,22 @@ def safe_lower(txt):
         return ''
 
 
+def safe_fnmatch(f, pattern):
+    """ fnmatch will fail if the pattern contains any of it's
+        key characters, like [, ] or !.
+    """
+    try:
+        return fnmatch.fnmatch(f, pattern)
+    except re.error:
+        return False
+
+
 def globber(path, pattern=u'*'):
     """ Return matching base file/folder names in folder `path` """
     # Cannot use glob.glob() because it doesn't support Windows long name notation
     if os.path.exists(path):
-        return [f for f in os.listdir(path) if fnmatch.fnmatch(f, pattern)]
-    else:
-        return []
+        return [f for f in os.listdir(path) if safe_fnmatch(f, pattern)]
+    return []
 
 
 def globber_full(path, pattern=u'*'):
@@ -114,13 +138,12 @@ def globber_full(path, pattern=u'*'):
     # Cannot use glob.glob() because it doesn't support Windows long name notation
     if os.path.exists(path):
         try:
-            return [os.path.join(path, f) for f in os.listdir(path) if fnmatch.fnmatch(f, pattern)]
+            return [os.path.join(path, f) for f in os.listdir(path) if safe_fnmatch(f, pattern)]
         except UnicodeDecodeError:
             # This happens on Linux when names are incorrectly encoded, retry using a non-Unicode path
             path = path.encode('utf-8')
-            return [os.path.join(path, f) for f in os.listdir(path) if fnmatch.fnmatch(f, pattern)]
-    else:
-        return []
+            return [os.path.join(path, f) for f in os.listdir(path) if safe_fnmatch(f, pattern)]
+    return []
 
 
 def cat_to_opts(cat, pp=None, script=None, priority=None):
@@ -147,12 +170,12 @@ def cat_to_opts(cat, pp=None, script=None, priority=None):
         if safe_lower(script) in ('', 'default'):
             script = def_cat.script()
 
-    if priority is None or priority == DEFAULT_PRIORITY:
+    if priority is None or priority == '' or priority == DEFAULT_PRIORITY:
         priority = my_cat.priority()
         if priority == DEFAULT_PRIORITY:
             priority = def_cat.priority()
 
-    # logging.debug('Cat->Attrib cat=%s pp=%s script=%s prio=%s', cat, pp, script, priority)
+    logging.debug('Cat->Attrib cat=%s pp=%s script=%s prio=%s', cat, pp, script, priority)
     return cat, pp, script, priority
 
 
@@ -226,48 +249,58 @@ _DEVICES = ('con', 'prn', 'aux', 'nul',
             'com1', 'com2', 'com3', 'com4', 'com5', 'com6', 'com7', 'com8', 'com9',
             'lpt1', 'lpt2', 'lpt3', 'lpt4', 'lpt5', 'lpt6', 'lpt7', 'lpt8', 'lpt9')
 
+
 def replace_win_devices(name):
-    ''' Remove reserved Windows device names from a name.
+    """ Remove reserved Windows device names from a name.
         aux.txt ==> _aux.txt
         txt.aux ==> txt.aux
-    '''
+    """
     if name:
         lname = name.lower()
         for dev in _DEVICES:
             if lname == dev or lname.startswith(dev + '.'):
                 name = '_' + name
                 break
+
+        # Remove special NTFS filename
+        if lname.startswith('$mft'):
+            name = name.replace('$', 'S', 1)
+
     return name
 
 
 def has_win_device(p):
     """ Return True if filename part contains forbidden name
+        Before and after sanitizing
     """
     p = os.path.split(p)[1].lower()
     for dev in _DEVICES:
-        if p == dev or p.startswith(dev + '.'):
+        if p == dev or p.startswith(dev + '.') or p.startswith('_' + dev + '.'):
             return True
     return False
 
 
-if sabnzbd.WIN32:
-    # the colon should be here too, but we'll handle that separately
-    CH_ILLEGAL = r'\/<>?*|"'
-    CH_LEGAL = r'++{}!@#`'
-else:
-    CH_ILLEGAL = r'/'
-    CH_LEGAL = r'+'
+CH_ILLEGAL = '/'
+CH_LEGAL = '+'
+CH_ILLEGAL_WIN = '\/<>?*|"\t:'
+CH_LEGAL_WIN = '++{}!@#`+;'
 
 
-def sanitize_filename(name, allow_win_devices=False):
+def sanitize_filename(name):
     """ Return filename with illegal chars converted to legal ones
         and with the par2 extension always in lowercase
     """
     if not name:
         return name
+
     illegal = CH_ILLEGAL
     legal = CH_LEGAL
 
+    if sabnzbd.WIN32 or cfg.sanitize_safe():
+        # Remove all bad Windows chars too
+        illegal += CH_ILLEGAL_WIN
+        legal += CH_LEGAL_WIN
+
     if ':' in name:
         if sabnzbd.WIN32:
             # Compensate for the odd way par2 on Windows substitutes a colon character
@@ -276,9 +309,6 @@ def sanitize_filename(name, allow_win_devices=False):
             # Compensate for the foolish way par2 on OSX handles a colon character
             name = name[name.rfind(':') + 1:]
 
-    if sabnzbd.WIN32 and not allow_win_devices:
-        name = replace_win_devices(name)
-
     lst = []
     for ch in name.strip():
         if ch in illegal:
@@ -286,6 +316,9 @@ def sanitize_filename(name, allow_win_devices=False):
         lst.append(ch)
     name = ''.join(lst)
 
+    if sabnzbd.WIN32 or cfg.sanitize_safe():
+        name = replace_win_devices(name)
+
     if not name:
         name = 'unknown'
 
@@ -315,10 +348,10 @@ def sanitize_foldername(name, limit=True):
         illegal = FL_ILLEGAL
         legal = FL_LEGAL
 
-    if cfg.sanitize_safe():
+    if sabnzbd.WIN32 or cfg.sanitize_safe():
         # Remove all bad Windows chars too
-        illegal += r'\/<>?*|":'
-        legal += r'++{}!@#`;'
+        illegal += CH_ILLEGAL_WIN
+        legal += CH_LEGAL_WIN
 
     repl = cfg.replace_illegal()
     lst = []
@@ -330,12 +363,7 @@ def sanitize_foldername(name, limit=True):
         else:
             lst.append(ch)
     name = ''.join(lst)
-
     name = name.strip()
-    if name != '.' and name != '..':
-        name = name.rstrip('.')
-    if not name:
-        name = 'unknown'
 
     if sabnzbd.WIN32 or cfg.sanitize_safe():
         name = replace_win_devices(name)
@@ -344,6 +372,12 @@ def sanitize_foldername(name, limit=True):
     if limit and len(name) > maxlen:
         name = name[:maxlen]
 
+    # And finally, make sure it doesn't end in a dot
+    if name != '.' and name != '..':
+        name = name.rstrip('.')
+    if not name:
+        name = 'unknown'
+
     return name
 
 
@@ -383,6 +417,7 @@ def sanitize_files_in_folder(folder):
             new_path = os.path.join(root, sanitize_filename(file_))
             if path != new_path:
                 try:
+                    logging.debug('Filename-sanitizer will rename %s to %s', path, new_path)
                     os.rename(path, new_path)
                     path = new_path
                 except:
@@ -391,20 +426,11 @@ def sanitize_files_in_folder(folder):
     return lst
 
 
-def flag_file(path, flag, create=False):
-    """ Create verify flag file or return True if it already exists """
-    path = os.path.join(path, JOB_ADMIN)
-    path = os.path.join(path, flag)
-    if create:
-        try:
-            f = open(path, 'w')
-            f.write('ok\n')
-            f.close()
-            return True
-        except IOError:
-            return False
-    else:
-        return os.path.exists(path)
+def is_obfuscated_filename(filename):
+    """ Check if this file has an extension, if not, it's
+        probably obfuscated and we don't use it
+    """
+    return os.path.splitext(filename)[1] == ''
 
 
 ##############################################################################
@@ -496,16 +522,16 @@ def create_real_path(name, loc, path, umask=False, writable=True):
             logging.info('%s directory: %s does not exist, try to create it', name, my_dir)
             if not create_all_dirs(my_dir, umask):
                 logging.error(T('Cannot create directory %s'), clip_path(my_dir))
-                return (False, my_dir)
+                return False, my_dir
 
         checks = (os.W_OK + os.R_OK) if writable else os.R_OK
         if os.access(my_dir, checks):
-            return (True, my_dir)
+            return True, my_dir
         else:
             logging.error(T('%s directory: %s error accessing'), name, clip_path(my_dir))
-            return (False, my_dir)
+            return False, my_dir
     else:
-        return (False, "")
+        return False, ""
 
 
 def is_relative_path(p):
@@ -719,13 +745,12 @@ def from_units(val):
         return 0.0
 
 
-def to_units(val, spaces=0, dec_limit=2, postfix=''):
+def to_units(val, spaces=0, postfix=''):
     """ Convert number to K/M/G/T/P notation
         Add "spaces" if not ending in letter
-        dig_limit==1 show single decimal for M and higher
-        dig_limit==2 show single decimal for G and higher
+        Show single decimal for M and higher
     """
-    decimals = 0
+    dec_limit = 1
     if val < 0:
         sign = '-'
     else:
@@ -752,6 +777,31 @@ def to_units(val, spaces=0, dec_limit=2, postfix=''):
     return fmt % (sign, val, unit, postfix)
 
 
+def caller_name(skip=2):
+    """Get a name of a caller in the format module.method
+       Originally used: https://gist.github.com/techtonik/2151727
+       Adapted for speed by using sys calls directly
+    """
+    # Only do the tracing on Debug (function is always called)
+    if cfg.log_level() != 2:
+        return 'N/A'
+
+    parentframe = sys._getframe(skip)
+    function_name = parentframe.f_code.co_name
+
+    # Modulename not available in the binaries, we can use the filename instead
+    if getattr(sys, 'frozen', None):
+        module_name = inspect.getfile(parentframe)
+    else:
+        module_name = inspect.getmodule(parentframe).__name__
+
+    # For decorated functions we have to go deeper
+    if function_name in ('call_func', 'wrap') and skip == 2:
+        return caller_name(4)
+
+    return ".".join([module_name, function_name])
+
+
 def same_file(a, b):
     """ Return 0 if A and B have nothing in common
         return 1 if A and B are actually the same path
@@ -780,7 +830,7 @@ def exit_sab(value):
     sys.stdout.flush()
     if getattr(sys, 'frozen', None) == 'macosx_app':
         sabnzbd.SABSTOP = True
-        from PyObjCTools import AppHelper  # @UnresolvedImport
+        from PyObjCTools import AppHelper
         AppHelper.stopEventLoop()
     sys.exit(value)
 
@@ -799,7 +849,15 @@ def split_host(srv):
         port = int(port)
     except:
         port = None
-    return (host, port)
+    return host, port
+
+
+def get_from_url(url):
+    """ Retrieve URL and return content """
+    try:
+        return urllib2.urlopen(url).read()
+    except:
+        return None
 
 
 def check_mount(path):
@@ -823,33 +881,79 @@ def check_mount(path):
 
 
 def get_cache_limit():
-    """ Depending on OS, calculate cache limit """
-    # OSX/Windows use Default value
-    if sabnzbd.WIN32 or sabnzbd.DARWIN:
-        return DEF_CACHE_LIMIT
-
+    """ Depending on OS, calculate cache limits.
+        In ArticleCache it will make sure we stay
+        within system limits for 32/64 bit
+    """
     # Calculate, if possible
     try:
+        if sabnzbd.WIN32:
+            # Windows
+            mem_bytes = get_windows_memory()
+        elif sabnzbd.DARWIN:
+            # macOS
+            mem_bytes = get_darwin_memory()
+        else:
+            # Linux
+            mem_bytes = (os.sysconf('SC_PAGE_SIZE') * os.sysconf('SC_PHYS_PAGES'))
+
         # Use 1/4th of available memory
-        mem_bytes = (os.sysconf('SC_PAGE_SIZE') * os.sysconf('SC_PHYS_PAGES'))/4
-        # Not more than the maximum we think is reasonable
-        if mem_bytes > from_units(DEF_CACHE_LIMIT):
-            return DEF_CACHE_LIMIT
-        elif mem_bytes > from_units('32M'):
-            # We make sure it's at least a valid value
+        mem_bytes = mem_bytes/4
+
+        # We don't want to set a value that's too high
+        if mem_bytes > from_units(DEF_ARTICLE_CACHE_MAX):
+            return DEF_ARTICLE_CACHE_MAX
+
+        # We make sure it's at least a valid value
+        if mem_bytes > from_units('32M'):
             return to_units(mem_bytes)
     except:
         pass
-    # If failed, leave empty so user needs to decide
+
+    # Always at least minimum on Windows/macOS
+    if sabnzbd.WIN32 and sabnzbd.DARWIN:
+        return DEF_ARTICLE_CACHE_DEFAULT
+
+    # If failed, leave empty for Linux so user needs to decide
     return ''
 
 
+def get_windows_memory():
+    """ Use ctypes to extract available memory """
+    class MEMORYSTATUSEX(ctypes.Structure):
+        _fields_ = [
+            ("dwLength", ctypes.c_ulong),
+            ("dwMemoryLoad", ctypes.c_ulong),
+            ("ullTotalPhys", ctypes.c_ulonglong),
+            ("ullAvailPhys", ctypes.c_ulonglong),
+            ("ullTotalPageFile", ctypes.c_ulonglong),
+            ("ullAvailPageFile", ctypes.c_ulonglong),
+            ("ullTotalVirtual", ctypes.c_ulonglong),
+            ("ullAvailVirtual", ctypes.c_ulonglong),
+            ("sullAvailExtendedVirtual", ctypes.c_ulonglong),
+        ]
+
+        def __init__(self):
+            # have to initialize this to the size of MEMORYSTATUSEX
+            self.dwLength = ctypes.sizeof(self)
+            super(MEMORYSTATUSEX, self).__init__()
+
+    stat = MEMORYSTATUSEX()
+    ctypes.windll.kernel32.GlobalMemoryStatusEx(ctypes.byref(stat))
+    return stat.ullTotalPhys
+
+
+def get_darwin_memory():
+    """ Use system-call to extract total memory on macOS """
+    system_output = sabnzbd.newsunpack.run_simple(['sysctl', 'hw.memsize'])
+    return float(system_output.split()[1])
+
+
 ##############################################################################
-# Locked directory operations
+# Locked directory operations to avoid problems with simultaneous add/remove
 ##############################################################################
 DIR_LOCK = threading.RLock()
 
-
 @synchronized(DIR_LOCK)
 def get_unique_path(dirpath, n=0, create_dir=True):
     """ Determine a unique folder or filename """
@@ -872,14 +976,16 @@ def get_unique_path(dirpath, n=0, create_dir=True):
 
 @synchronized(DIR_LOCK)
 def get_unique_filename(path):
-    """ Check if path is unique. If not, add number like: "/path/name.NUM.ext". """
+    """ Check if path is unique.
+        If not, add number like: "/path/name.NUM.ext".
+    """
     num = 1
+    new_path, fname = os.path.split(path)
+    name, ext = os.path.splitext(fname)
     while os.path.exists(path):
-        path, fname = os.path.split(path)
-        name, ext = os.path.splitext(fname)
         fname = "%s.%d%s" % (name, num, ext)
         num += 1
-        path = os.path.join(path, fname)
+        path = os.path.join(new_path, fname)
     return path
 
 
@@ -894,6 +1000,22 @@ def create_dirs(dirpath):
     return dirpath
 
 
+@synchronized(DIR_LOCK)
+def recursive_listdir(dir):
+    """ List all files in dirs and sub-dirs """
+    filelist = []
+    for root, dirs, files in os.walk(dir):
+        for file in files:
+            if '.AppleDouble' not in root and '.DS_Store' not in root:
+                try:
+                    p = os.path.join(root, file)
+                    filelist.append(p)
+                except UnicodeDecodeError:
+                    # Just skip failing names
+                    pass
+    return filelist
+
+
 @synchronized(DIR_LOCK)
 def move_to_path(path, new_path):
     """ Move a file to a new path, optionally give unique filename
@@ -904,15 +1026,14 @@ def move_to_path(path, new_path):
     new_path = os.path.abspath(new_path)
     if overwrite and os.path.exists(new_path):
         try:
-            os.remove(new_path)
+            remove_file(new_path)
         except:
             overwrite = False
     if not overwrite:
         new_path = get_unique_filename(new_path)
 
     if new_path:
-        logging.debug("Moving. Old path: %s New path: %s Overwrite: %s",
-                                                  path, new_path, overwrite)
+        logging.debug("Moving (overwrite: %s) %s => %s", overwrite, path, new_path)
         try:
             # First try cheap rename
             renamer(path, new_path)
@@ -923,7 +1044,7 @@ def move_to_path(path, new_path):
                 if not os.path.exists(os.path.dirname(new_path)):
                     create_dirs(os.path.dirname(new_path))
                 shutil.copyfile(path, new_path)
-                os.remove(path)
+                remove_file(path)
             except:
                 # Check if the old-file actually exists (possible delete-delays)
                 if not os.path.exists(path):
@@ -964,11 +1085,8 @@ def get_filepath(path, nzo, filename):
     # It does no umask setting
     # It uses the dir_lock for the (rare) case that the
     # download_dir is equal to the complete_dir.
-    dirname = nzo.work_name
-    created = nzo.created
-
-    dName = dirname
-    if not created:
+    dName = dirname = nzo.work_name
+    if not nzo.created:
         for n in xrange(200):
             dName = dirname
             if n:
@@ -997,6 +1115,98 @@ def get_filepath(path, nzo, filename):
     return fullPath
 
 
+@synchronized(DIR_LOCK)
+def renamer(old, new):
+    """ Rename file/folder with retries for Win32 """
+    # Sanitize last part of new name
+    path, name = os.path.split(new)
+    # Use the more stringent folder rename to end up with a nicer name,
+    # but do not trim size
+    new = os.path.join(path, sanitize_foldername(name, False))
+
+    logging.debug('Renaming "%s" to "%s"', old, new)
+    if sabnzbd.WIN32:
+        retries = 15
+        while retries > 0:
+            # First we try 3 times with os.rename
+            if retries > 12:
+                try:
+                    os.rename(old, new)
+                    return
+                except:
+                    retries -= 1
+                    time.sleep(3)
+                    continue
+
+            # Now we try the back-up method
+            logging.debug('Could not rename, trying move for %s to %s', old, new)
+            try:
+                shutil.move(old, new)
+                return
+            except WindowsError, err:
+                logging.debug('Error renaming "%s" to "%s" <%s>', old, new, err)
+                if err[0] == 32:
+                    logging.debug('Retry rename %s to %s', old, new)
+                    retries -= 1
+                else:
+                    raise WindowsError(err)
+            time.sleep(3)
+        raise WindowsError(err)
+    else:
+        shutil.move(old, new)
+
+
+@synchronized(DIR_LOCK)
+def remove_dir(path):
+    """ Remove directory with retries for Win32 """
+    logging.debug('[%s] Deleting dir %s', caller_name(), path)
+    if sabnzbd.WIN32:
+        retries = 15
+        while retries > 0:
+            try:
+                os.rmdir(path)
+                return
+            except WindowsError, err:
+                if err[0] == 32:
+                    logging.debug('Retry delete %s', path)
+                    retries -= 1
+                else:
+                    raise WindowsError(err)
+            time.sleep(3)
+        raise WindowsError(err)
+    else:
+        os.rmdir(path)
+
+
+@synchronized(DIR_LOCK)
+def remove_all(path, pattern='*', keep_folder=False, recursive=False):
+    """ Remove folder and all its content (optionally recursive) """
+    if os.path.exists(path):
+        files = globber_full(path, pattern)
+        if pattern == '*' and not sabnzbd.WIN32:
+            files.extend(globber_full(path, '.*'))
+
+        for f in files:
+            if os.path.isfile(f):
+                try:
+                    remove_file(f)
+                except:
+                    logging.info('Cannot remove file %s', f)
+            elif recursive:
+                remove_all(f, pattern, False, True)
+        if not keep_folder:
+            try:
+                remove_dir(path)
+            except:
+                logging.info('Cannot remove folder %s', path)
+
+
+def remove_file(path):
+    """ Wrapper function so any file removal is logged """
+    logging.debug('[%s] Deleting file %s', caller_name(), path)
+    os.remove(path)
+
+
 def trim_win_path(path):
     """ Make sure Windows path stays below 70 by trimming last part """
     if sabnzbd.WIN32 and len(path) > 69:
@@ -1008,15 +1218,6 @@ def trim_win_path(path):
     return path
 
 
-def check_win_maxpath(folder):
-    """ Return False if any file path in folder exceeds the Windows maximum """
-    if sabnzbd.WIN32:
-        for p in os.listdir(folder):
-            if len(os.path.join(folder, p)) > 259:
-                return False
-    return True
-
-
 def make_script_path(script):
     """ Return full script path, if any valid script exists, else None """
     s_path = None
@@ -1039,6 +1240,14 @@ def get_admin_path(name, future):
         return os.path.join(os.path.join(cfg.download_dir.get_path(), name), JOB_ADMIN)
 
 
+def is_rarfile(rarfile_path):
+    """ Wrapper in case it crashes due to missing file or long-path problems """
+    try:
+        return rarfile.is_rarfile(rarfile_path)
+    except:
+        return False
+
+
 def on_cleanup_list(filename, skip_nzb=False):
     """ Return True if a filename matches the clean-up list """
     lst = cfg.cleanup_list()
@@ -1084,8 +1293,8 @@ def memory_usage():
     except:
         logging.debug('Error retrieving memory usage')
         logging.info("Traceback: ", exc_info=True)
-    else:
-        return ''
+
+
 try:
     _PAGE_SIZE = os.sysconf("SC_PAGE_SIZE")
 except:
@@ -1197,24 +1406,40 @@ else:
             return 20.0, 10.0
 
 
-__LAST_DISK_RESULT = {}
-__LAST_DISK_CALL = {}
-def diskspace(_dir, force=False):
+# Store all results to speed things up
+__DIRS_CHECKED = []
+__DISKS_SAME = None
+__LAST_DISK_RESULT = {'download_dir': [], 'complete_dir': []}
+__LAST_DISK_CALL = 0
+
+def diskspace(force=False):
     """ Wrapper to cache results """
-    if _dir not in __LAST_DISK_RESULT:
-        __LAST_DISK_RESULT[_dir] = [0.0, 0.0]
-        __LAST_DISK_CALL[_dir] = 0.0
+    global __DIRS_CHECKED, __DISKS_SAME, __LAST_DISK_RESULT, __LAST_DISK_CALL
+
+    # Reset everything when folders changed
+    dirs_to_check = [cfg.download_dir.get_path(), cfg.complete_dir.get_path()]
+    if __DIRS_CHECKED != dirs_to_check:
+        __DIRS_CHECKED = dirs_to_check
+        __DISKS_SAME = None
+        __LAST_DISK_RESULT = {'download_dir': [], 'complete_dir': []}
+        __LAST_DISK_CALL = 0
 
     # When forced, ignore any cache to avoid problems in UI
     if force:
-        return diskspace_base(_dir)
+        __LAST_DISK_CALL = 0
 
     # Check against cache
-    if time.time() > __LAST_DISK_CALL[_dir] + 10.0:
-        __LAST_DISK_RESULT[_dir] = diskspace_base(_dir)
-        __LAST_DISK_CALL[_dir] = time.time()
+    if time.time() > __LAST_DISK_CALL + 10.0:
+        # Same disk? Then copy-paste
+        __LAST_DISK_RESULT['download_dir'] = diskspace_base(cfg.download_dir.get_path())
+        __LAST_DISK_RESULT['complete_dir'] = __LAST_DISK_RESULT['download_dir'] if __DISKS_SAME else diskspace_base(cfg.complete_dir.get_path())
+        __LAST_DISK_CALL = time.time()
 
-    return __LAST_DISK_RESULT[_dir]
+    # Do we know if it's same disk?
+    if __DISKS_SAME is None:
+        __DISKS_SAME = (__LAST_DISK_RESULT['download_dir'] == __LAST_DISK_RESULT['complete_dir'])
+
+    return __LAST_DISK_RESULT
 
 
 ##############################################################################
@@ -1230,7 +1455,7 @@ def create_https_certificates(ssl_cert, ssl_key):
     try:
         from sabnzbd.utils.certgen import generate_key, generate_local_cert
         private_key = generate_key(key_size=2048, output_file=ssl_key)
-        cert = generate_local_cert(private_key, days_valid=3560, output_file=ssl_cert, LN=u'SABnzbd', ON=u'SABnzbd', CN=u'localhost')
+        generate_local_cert(private_key, days_valid=3560, output_file=ssl_cert, LN=u'SABnzbd', ON=u'SABnzbd')
         logging.info('Self-signed certificates generated successfully')
     except:
         logging.error(T('Error creating SSL key and certificate'))
@@ -1252,6 +1477,7 @@ def get_all_passwords(nzo):
     pw = nzo.nzo_info.get('password')
     if pw:
         meta_passwords.append(pw)
+
     if meta_passwords:
         if nzo.password == meta_passwords[0]:
             # this nzo.password came from meta, so don't use it twice
@@ -1259,19 +1485,23 @@ def get_all_passwords(nzo):
         else:
             passwords.extend(meta_passwords)
         logging.info('Read %s passwords from meta data in NZB: %s', len(meta_passwords), meta_passwords)
+
     pw_file = cfg.password_file.get_path()
     if pw_file:
         try:
-            pwf = open(pw_file, 'r')
-            lines = pwf.read().split('\n')
+            with open(pw_file, 'r') as pwf:
+                lines = pwf.read().split('\n')
             # Remove empty lines and space-only passwords and remove surrounding spaces
             pws = [pw.strip('\r\n ') for pw in lines if pw.strip('\r\n ')]
             logging.debug('Read these passwords from file: %s', pws)
             passwords.extend(pws)
-            pwf.close()
             logging.info('Read %s passwords from file %s', len(pws), pw_file)
-        except IOError:
-            logging.info('Failed to read the passwords file %s', pw_file)
+
+            # Check size
+            if len(pws) > 30:
+                logging.warning(T('Your password file contains more than 30 passwords, testing all these passwords takes a lot of time. Try to only list useful passwords.'))
+        except:
+            logging.warning('Failed to read the passwords file %s', pw_file)
 
     if nzo.password:
         # If an explicit password was set, add a retry without password, just in case.
@@ -1301,8 +1531,22 @@ def find_on_path(targets):
     return None
 
 
-_RE_IP4 = re.compile(r'inet\s+(addr:\s*){0,1}(\d+\.\d+\.\d+\.\d+)')
-_RE_IP6 = re.compile(r'inet6\s+(addr:\s*){0,1}([0-9a-f:]+)', re.I)
+def probablyipv4(ip):
+    if ip.count('.') == 3 and re.sub('[0123456789.]', '', ip) == '':
+        return True
+    else:
+        return False
+
+
+def probablyipv6(ip):
+    # Returns True if the given input is probably an IPv6 address
+    # Square Brackets like '[2001::1]' are OK
+    if ip.count(':') >= 2 and re.sub('[0123456789abcdefABCDEF:\[\]]', '', ip) == '':
+        return True
+    else:
+        return False
+
+
 def ip_extract():
     """ Return list of IP addresses of this system """
     ips = []
@@ -1329,96 +1573,14 @@ def ip_extract():
         output = p.stdout.read()
         p.wait()
         for line in output.split('\n'):
-            m = _RE_IP4.search(line)
+            m = RE_IP4.search(line)
             if not (m and m.group(2)):
-                m = _RE_IP6.search(line)
+                m = RE_IP6.search(line)
             if m and m.group(2):
                 ips.append(m.group(2))
     return ips
 
 
-def renamer(old, new):
-    """ Rename file/folder with retries for Win32 """
-    # Sanitize last part of new name
-    path, name = os.path.split(new)
-    # Use the more stringent folder rename to end up with a nicer name,
-    # but do not trim size
-    new = os.path.join(path, sanitize_foldername(name, False))
-
-    logging.debug('Renaming "%s" to "%s"', old, new)
-    if sabnzbd.WIN32:
-        retries = 15
-        while retries > 0:
-            # First we try 3 times with os.rename
-            if retries > 12:
-                try:
-                    os.rename(old, new)
-                    return
-                except:
-                    retries -= 1
-                    time.sleep(3)
-                    continue
-
-            # Now we try the back-up method
-            logging.debug('Could not rename, trying move for %s to %s', old, new)
-            try:
-                shutil.move(old, new)
-                return
-            except WindowsError, err:
-                logging.debug('Error renaming "%s" to "%s" <%s>', old, new, err)
-                if err[0] == 32:
-                    logging.debug('Retry rename %s to %s', old, new)
-                    retries -= 1
-                else:
-                    raise WindowsError(err)
-            time.sleep(3)
-        raise WindowsError(err)
-    else:
-        shutil.move(old, new)
-
-
-def remove_dir(path):
-    """ Remove directory with retries for Win32 """
-    if sabnzbd.WIN32:
-        retries = 15
-        while retries > 0:
-            try:
-                os.rmdir(path)
-                return
-            except WindowsError, err:
-                if err[0] == 32:
-                    logging.debug('Retry delete %s', path)
-                    retries -= 1
-                else:
-                    raise WindowsError(err)
-            time.sleep(3)
-        raise WindowsError(err)
-    else:
-        os.rmdir(path)
-
-
-def remove_all(path, pattern='*', keep_folder=False, recursive=False):
-    """ Remove folder and all its content (optionally recursive) """
-    if os.path.exists(path):
-        files = globber_full(path, pattern)
-        if pattern == '*' and not sabnzbd.WIN32:
-            files.extend(globber_full(path, '.*'))
-
-        for f in files:
-            if os.path.isfile(f):
-                try:
-                    os.remove(f)
-                except:
-                    logging.info('Cannot remove file %s', f)
-            elif recursive:
-                remove_all(f, pattern, False, True)
-        if not keep_folder:
-            try:
-                os.rmdir(path)
-            except:
-                logging.info('Cannot remove folder %s', path)
-
-
 def is_writable(path):
     """ Return True is file is writable (also when non-existent) """
     if os.path.isfile(path):
@@ -1427,15 +1589,6 @@ def is_writable(path):
         return True
 
 
-def format_source_url(url):
-    """ Format URL suitable for 'Source' stage """
-    if sabnzbd.HAVE_SSL:
-        prot = 'https'
-    else:
-        prot = 'http:'
-    return url
-
-
 def get_base_url(url):
     """ Return only the true root domain for the favicon, so api.oznzb.com -> oznzb.com
         But also api.althub.co.za -> althub.co.za
@@ -1474,6 +1627,7 @@ def starts_with_path(path, prefix):
 def set_chmod(path, permissions, report):
     """ Set 'permissions' on 'path', report any errors when 'report' is True """
     try:
+        logging.debug('Applying permissions %s (octal) to %s', oct(permissions), path)
         os.chmod(path, permissions)
     except:
         lpath = path.lower()
@@ -1549,3 +1703,11 @@ def get_urlbase(url):
     """ Return the base URL (like http://server.domain.com/) """
     parsed_uri = urlparse(url)
     return '{uri.scheme}://{uri.netloc}/'.format(uri=parsed_uri)
+
+
+def nntp_to_msg(text):
+    """ Format raw NNTP data for display """
+    if isinstance(text, list):
+        text = text[0]
+    lines = text.split('\r\n')
+    return lines[0]
