@@ -1,5 +1,5 @@
 #!/usr/bin/python -OO
-# Copyright 2008-2017 The SABnzbd-Team <team@sabnzbd.org>
+# Copyright 2007-2019 The SABnzbd-Team <team@sabnzbd.org>
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -36,7 +36,7 @@ import logging
 import sabnzbd
 import sabnzbd.cfg
 
-from sabnzbd.constants import VALID_ARCHIVES, MEBI, Status
+from sabnzbd.constants import VALID_ARCHIVES, VALID_NZB_FILES, MEBI, Status
 from sabnzbd.misc import get_filename, get_ext, diskspace, to_units
 from sabnzbd.panic import launch_a_browser
 import sabnzbd.notifier as notifier
@@ -208,7 +208,7 @@ class SABnzbdDelegate(NSObject):
 
         for speed in sorted(speeds.keys()):
             menu_speed_item = NSMenuItem.alloc().initWithTitle_action_keyEquivalent_('%s' % (speeds[speed]), 'speedlimitAction:', '')
-            menu_speed_item.setRepresentedObject_("%s" % (speed))
+            menu_speed_item.setRepresentedObject_("%s" % speed)
             self.menu_speed.addItem_(menu_speed_item)
 
         self.speed_menu_item.setSubmenu_(self.menu_speed)
@@ -414,7 +414,7 @@ class SABnzbdDelegate(NSObject):
                     if history['status'] != Status.COMPLETED:
                         jobfailed = NSAttributedString.alloc().initWithString_attributes_(job, self.failedAttributes)
                         menu_history_item.setAttributedTitle_(jobfailed)
-                    menu_history_item.setRepresentedObject_("%s" % (path))
+                    menu_history_item.setRepresentedObject_("%s" % path)
                     self.menu_history.addItem_(menu_history_item)
             else:
                 menu_history_item = NSMenuItem.alloc().initWithTitle_action_keyEquivalent_(T('Empty'), '', '')
@@ -461,7 +461,7 @@ class SABnzbdDelegate(NSObject):
                     self.setMenuTitle_("")
             elif bytes_left > 0:
                 self.state = ""
-                speed = to_units(bpsnow, dec_limit=1)
+                speed = to_units(bpsnow)
                 # "10.1 MB/s" doesn't fit, remove space char
                 if 'M' in speed and len(speed) > 5:
                     speed = speed.replace(' ', '')
@@ -483,9 +483,9 @@ class SABnzbdDelegate(NSObject):
             if self.state != "" and self.info != "":
                 self.state_menu_item.setTitle_("%s - %s" % (self.state, self.info))
             if self.info == "":
-                self.state_menu_item.setTitle_("%s" % (self.state))
+                self.state_menu_item.setTitle_("%s" % self.state)
             else:
-                self.state_menu_item.setTitle_("%s" % (self.info))
+                self.state_menu_item.setTitle_("%s" % self.info)
 
         except:
             logging.info("[osx] stateUpdate Exception %s" % (sys.exc_info()[0]))
@@ -618,8 +618,8 @@ class SABnzbdDelegate(NSObject):
 
     def diskspaceUpdate(self):
         try:
-            self.completefolder_menu_item.setTitle_("%s%.2f GB" % (T('Complete Folder') + '\t\t\t', diskspace(sabnzbd.cfg.complete_dir.get_path())[1]))
-            self.incompletefolder_menu_item.setTitle_("%s%.2f GB" % (T('Incomplete Folder') + '\t\t', diskspace(sabnzbd.cfg.download_dir.get_path())[1]))
+            self.completefolder_menu_item.setTitle_("%s%.2f GB" % (T('Complete Folder') + '\t\t\t', diskspace()['complete_dir'][1]))
+            self.incompletefolder_menu_item.setTitle_("%s%.2f GB" % (T('Incomplete Folder') + '\t\t', diskspace()['download_dir'][1]))
         except:
             logging.info("[osx] diskspaceUpdate Exception %s" % (sys.exc_info()[0]))
 
@@ -721,6 +721,7 @@ class SABnzbdDelegate(NSObject):
 
     def restartAction_(self, sender):
         self.setMenuTitle_("\n\n%s\n" % (T('Stopping...')))
+        logging.info('Restart requested by tray')
         sabnzbd.trigger_restart()
         self.setMenuTitle_("\n\n%s\n" % (T('Stopping...')))
 
@@ -753,7 +754,7 @@ class SABnzbdDelegate(NSObject):
                     if get_ext(name) in VALID_ARCHIVES:
                         # logging.info('[osx] archive')
                         dirscanner.ProcessArchiveFile(fn, name, keep=True)
-                    elif get_ext(name) in ('.nzb', '.gz', '.bz2'):
+                    elif get_ext(name) in VALID_NZB_FILES:
                         # logging.info('[osx] nzb')
                         dirscanner.ProcessSingleFile(fn, name, keep=True)
         # logging.info('opening done')
@@ -763,10 +764,7 @@ class SABnzbdDelegate(NSObject):
         self.setMenuTitle_("\n\n%s\n" % (T('Stopping...')))
         self.status_item.setHighlightMode_(NO)
         self.osx_icon = False
-        logging.info('[osx] application stopping daemon')
-        sabnzbd.halt()
-        cherrypy.engine.exit()
-        sabnzbd.SABSTOP = True
+        sabnzbd.shutdown_program()
         try:
             notifier.send_notification('SABnzbd', T('SABnzbd shutdown finished'), notifier.NOTIFICATION['other'])
         except AttributeError:
