@@ -1,5 +1,5 @@
 #!/usr/bin/python -OO
-# Copyright 2008-2017 The SABnzbd-Team <team@sabnzbd.org>
+# Copyright 2007-2019 The SABnzbd-Team <team@sabnzbd.org>
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -24,7 +24,6 @@ import Queue
 import logging
 import sabnzbd
 import xml.sax.saxutils
-import xml.etree.ElementTree
 import time
 import re
 
@@ -32,14 +31,14 @@ from sabnzbd.newsunpack import unpack_magic, par2_repair, external_processing, \
     sfv_check, build_filelists, rar_sort
 from threading import Thread
 from sabnzbd.misc import real_path, get_unique_path, create_dirs, move_to_path, \
-    make_script_path, long_path, clip_path, \
+    make_script_path, long_path, clip_path, recursive_listdir, \
     on_cleanup_list, renamer, remove_dir, remove_all, globber, globber_full, \
-    set_permissions, cleanup_empty_directories, check_win_maxpath, fix_unix_encoding, \
-    sanitize_and_trim_path, sanitize_files_in_folder
-from sabnzbd.tvsort import Sorter
+    set_permissions, cleanup_empty_directories, fix_unix_encoding, \
+    sanitize_and_trim_path, sanitize_files_in_folder, remove_file
+from sabnzbd.sorting import Sorter
 from sabnzbd.constants import REPAIR_PRIORITY, TOP_PRIORITY, POSTPROC_QUEUE_FILE_NAME, \
     POSTPROC_QUEUE_VERSION, sample_match, JOB_ADMIN, Status, VERIFIED_FILE
-from sabnzbd.encoding import TRANS, unicoder, deunicode
+from sabnzbd.encoding import TRANS, unicoder
 from sabnzbd.rating import Rating
 import sabnzbd.emailer as emailer
 import sabnzbd.dirscanner as dirscanner
@@ -50,37 +49,46 @@ import sabnzbd.nzbqueue
 import sabnzbd.database as database
 import sabnzbd.notifier as notifier
 import sabnzbd.utils.rarfile as rarfile
+import sabnzbd.utils.checkdir
 
+MAX_FAST_JOB_COUNT = 3
+
+# Match samples
+RE_SAMPLE = re.compile(sample_match, re.I)
 
 class PostProcessor(Thread):
     """ PostProcessor thread, designed as Singleton """
     do = None  # Link to instance of the thread
 
-    def __init__(self, queue=None, history_queue=None):
-        """ Initialize, optionally passing existing queue """
+    def __init__(self):
+        """ Initialize PostProcessor thread """
         Thread.__init__(self)
 
         # This history queue is simply used to log what active items to display in the web_ui
-        if history_queue:
-            self.history_queue = history_queue
-        else:
-            self.load()
+        self.load()
 
         if self.history_queue is None:
             self.history_queue = []
 
-        if queue:
-            self.queue = queue
-        else:
-            self.queue = Queue.Queue()
-            for nzo in self.history_queue:
-                self.process(nzo)
+        # Fast-queue for jobs already finished by DirectUnpack
+        self.fast_queue = Queue.Queue()
+
+        # Regular queue for jobs that might need more attention
+        self.slow_queue = Queue.Queue()
+
+        # Load all old jobs
+        for nzo in self.history_queue:
+            self.process(nzo)
+
+        # Counter to not only process fast-jobs
+        self.__fast_job_count = 0
+
+        # State variables
         self.__stop = False
+        self.__busy = False
         self.paused = False
         PostProcessor.do = self
 
-        self.__busy = False  # True while a job is being processed
-
     def save(self):
         """ Save postproc queue """
         logging.info("Saving postproc queue")
@@ -120,10 +128,14 @@ class PostProcessor(Thread):
         """ Push on finished job in the queue """
         if nzo not in self.history_queue:
             self.history_queue.append(nzo)
-        self.queue.put(nzo)
+
+        # Fast-track if it has DirectUnpacked jobs or if it's still going
+        if nzo.direct_unpacker and (nzo.direct_unpacker.success_sets or not nzo.direct_unpacker.killed):
+            self.fast_queue.put(nzo)
+        else:
+            self.slow_queue.put(nzo)
         self.save()
-        # Update the last check time
-        sabnzbd.LAST_HISTORY_UPDATE = time.time()
+        sabnzbd.history_updated()
 
     def remove(self, nzo):
         """ Remove given nzo from the queue """
@@ -132,25 +144,27 @@ class PostProcessor(Thread):
         except:
             pass
         self.save()
-        # Update the last check time
-        sabnzbd.LAST_HISTORY_UPDATE = time.time()
+        sabnzbd.history_updated()
 
     def stop(self):
         """ Stop thread after finishing running job """
         self.__stop = True
-        self.queue.put(None)
+        self.slow_queue.put(None)
+        self.fast_queue.put(None)
 
     def cancel_pp(self, nzo_id):
         """ Change the status, so that the PP is canceled """
         for nzo in self.history_queue:
-            if nzo.nzo_id == nzo_id and nzo.pp_active:
-                nzo.pp_active = False
+            if nzo.nzo_id == nzo_id:
+                nzo.abort_direct_unpacker()
+                if nzo.pp_active:
+                    nzo.pp_active = False
                 return True
         return None
 
     def empty(self):
         """ Return True if pp queue is empty """
-        return self.queue.empty() and not self.__busy
+        return self.slow_queue.empty() and self.fast_queue.empty() and not self.__busy
 
     def get_queue(self):
         """ Return list of NZOs that still need to be processed """
@@ -164,9 +178,28 @@ class PostProcessor(Thread):
         return None
 
     def run(self):
-        """ Actual processing """
-        check_eoq = False
+        """ Postprocessor loop """
+        # First we do a dircheck
+        complete_dir = sabnzbd.cfg.complete_dir.get_path()
+        if sabnzbd.utils.checkdir.isFAT(complete_dir):
+            logging.warning(T('Completed Download Folder %s is on FAT file system, limiting maximum file size to 4GB') % complete_dir)
+        else:
+            logging.info("Completed Download Folder %s is not on FAT", complete_dir)
 
+        # Check on Windows if we have unicode-subprocess
+        if sabnzbd.WIN32:
+            try:
+                import subprocessww
+            except ImportError:
+                logging.warning(T('Module subprocessww missing. Expect problems with Unicoded file and directory names in downloads.'))
+
+        # Do a pruge of the history-items if it was set, just to be sure
+        history_db = database.HistoryDB()
+        history_db.auto_history_purge()
+        history_db.close()
+
+        # Start looping
+        check_eoq = False
         while not self.__stop:
             self.__busy = False
 
@@ -174,16 +207,28 @@ class PostProcessor(Thread):
                 time.sleep(5)
                 continue
 
+            # Something in the fast queue?
             try:
-                nzo = self.queue.get(timeout=1)
-                if 0: assert isinstance(nzo, sabnzbd.nzbstuff.NzbObject)
+                # Every few fast-jobs we should check allow a
+                # slow job so that they don't wait forever
+                if self.__fast_job_count >= MAX_FAST_JOB_COUNT and self.slow_queue.qsize():
+                    raise Queue.Empty
+
+                nzo = self.fast_queue.get(timeout=2)
+                self.__fast_job_count += 1
             except Queue.Empty:
-                if check_eoq:
-                    check_eoq = False
-                    handle_empty_queue()
+                # Try the slow queue
+                try:
+                    nzo = self.slow_queue.get(timeout=2)
+                    # Reset fast-counter
+                    self.__fast_job_count = 0
+                except Queue.Empty:
+                    # Check for empty queue
+                    if check_eoq:
+                        check_eoq = False
+                        handle_empty_queue()
+                    # No fast or slow jobs, better luck next loop!
                     continue
-                else:
-                    nzo = self.queue.get()
 
             # Stop job
             if not nzo:
@@ -211,6 +256,9 @@ class PostProcessor(Thread):
                 history_db.close()
                 nzo.purge_data(keep_basic=False, del_files=True)
 
+            # Processing done
+            nzo.pp_active = False
+
             self.remove(nzo)
             check_eoq = True
 
@@ -220,7 +268,6 @@ class PostProcessor(Thread):
 
 def process_job(nzo):
     """ Process one job """
-    if 0: assert isinstance(nzo, sabnzbd.nzbstuff.NzbObject) # Assert only for debug purposes
     start = time.time()
 
     # keep track of whether we can continue
@@ -234,10 +281,8 @@ def process_job(nzo):
     nzb_list = []
     # These need to be initialized in case of a crash
     workdir_complete = ''
-    postproc_time = 0
     script_log = ''
     script_line = ''
-    crash_msg = ''
 
     # Get the job flags
     nzo.save_attribs()
@@ -251,13 +296,6 @@ def process_job(nzo):
     # Get the NZB name
     filename = nzo.final_name
 
-    if cfg.allow_streaming() and not (flag_repair or flag_unpack or flag_delete):
-        # After streaming, force +D
-        nzo.set_pp(3)
-        nzo.status = Status.FAILED
-        nzo.save_attribs()
-        all_ok = False
-
     if nzo.fail_msg:  # Special case: aborted due to too many missing data
         nzo.status = Status.FAILED
         nzo.save_attribs()
@@ -266,7 +304,6 @@ def process_job(nzo):
         unpack_error = 1
 
     try:
-
         # Get the folder containing the download result
         workdir = nzo.downpath
         tmp_workdir_complete = None
@@ -274,7 +311,7 @@ def process_job(nzo):
         # if no files are present (except __admin__), fail the job
         if all_ok and len(globber(workdir)) < 2:
             if nzo.precheck:
-                _enough, ratio = nzo.check_quality()
+                _enough, ratio = nzo.check_availability_ratio()
                 req_ratio = float(cfg.req_completion_rate()) / 100.0
                 # Make sure that rounded ratio doesn't equal required ratio
                 # when it is actually below required
@@ -298,16 +335,12 @@ def process_job(nzo):
                 unpack_error = 1
 
         script = nzo.script
-        cat = nzo.cat
-
         logging.info('Starting Post-Processing on %s' +
                      ' => Repair:%s, Unpack:%s, Delete:%s, Script:%s, Cat:%s',
-                     filename, flag_repair, flag_unpack, flag_delete, script, cat)
+                     filename, flag_repair, flag_unpack, flag_delete, script, nzo.cat)
 
         # Set complete dir to workdir in case we need to abort
         workdir_complete = workdir
-        dirname = nzo.final_name
-        marker_file = None
 
         # Par processing, if enabled
         if all_ok and flag_repair:
@@ -316,6 +349,11 @@ def process_job(nzo):
                 # Try to get more par files
                 return False
 
+        # If we don't need extra par2, we can disconnect
+        if sabnzbd.nzbqueue.NzbQueue.do.actives(grabs=False) == 0 and cfg.autodisconnect():
+            # This was the last job, close server connections
+            sabnzbd.downloader.Downloader.do.disconnect()
+
         # Sanitize the resulting files
         if sabnzbd.WIN32:
             sanitize_files_in_folder(workdir)
@@ -325,64 +363,29 @@ def process_job(nzo):
             all_ok = all_ok and not par_error
 
         if all_ok:
+            # Fix encodings
             fix_unix_encoding(workdir)
-            one_folder = False
-            # Determine class directory
-            if cfg.create_group_folders():
-                complete_dir = addPrefixes(cfg.complete_dir.get_path(), nzo.dirprefix)
-                complete_dir = create_dirs(complete_dir)
+
+            # Use dirs generated by direct-unpacker
+            if nzo.direct_unpacker and nzo.direct_unpacker.unpack_dir_info:
+                tmp_workdir_complete, workdir_complete, file_sorter, one_folder, marker_file = nzo.direct_unpacker.unpack_dir_info
             else:
-                catdir = config.get_categories(cat).dir()
-                if catdir.endswith('*'):
-                    catdir = catdir.strip('*')
-                    one_folder = True
-                complete_dir = real_path(cfg.complete_dir.get_path(), catdir)
-            complete_dir = long_path(complete_dir)
-
-            # TV/Movie/Date Renaming code part 1 - detect and construct paths
-            if cfg.enable_meta():
-                file_sorter = Sorter(nzo, cat)
-            else:
-                file_sorter = Sorter(None, cat)
-            complete_dir = file_sorter.detect(dirname, complete_dir)
-            if file_sorter.sort_file:
-                one_folder = False
-
-            complete_dir = sanitize_and_trim_path(complete_dir)
-
-            if one_folder:
-                workdir_complete = create_dirs(complete_dir)
-            else:
-                workdir_complete = get_unique_path(os.path.join(complete_dir, dirname), create_dir=True)
-                marker_file = set_marker(workdir_complete)
-
-            if not workdir_complete or not os.path.exists(workdir_complete):
-                crash_msg = T('Cannot create final folder %s') % unicoder(os.path.join(complete_dir, dirname))
-                raise IOError
-
-            if cfg.folder_rename() and not one_folder:
-                tmp_workdir_complete = prefix(workdir_complete, '_UNPACK_')
-                try:
-                    renamer(workdir_complete, tmp_workdir_complete)
-                except:
-                    pass  # On failure, just use the original name
-            else:
-                tmp_workdir_complete = workdir_complete
+                # Generate extraction path
+                tmp_workdir_complete, workdir_complete, file_sorter, one_folder, marker_file = prepare_extraction_path(nzo)
 
             newfiles = []
             # Run Stage 2: Unpack
             if flag_unpack:
-                if all_ok:
-                    # set the current nzo status to "Extracting...". Used in History
-                    nzo.status = Status.EXTRACTING
-                    logging.info("Running unpack_magic on %s", filename)
-                    unpack_error, newfiles = unpack_magic(nzo, workdir, tmp_workdir_complete, flag_delete, one_folder, (), (), (), (), ())
-                    if sabnzbd.WIN32:
-                        # Sanitize the resulting files
-                        newfiles = sanitize_files_in_folder(tmp_workdir_complete)
-                    logging.info("unpack_magic finished on %s", filename)
-                else:
-                    nzo.set_unpack_info('Unpack', T('No post-processing because of failed verification'))
+                # set the current nzo status to "Extracting...". Used in History
+                nzo.status = Status.EXTRACTING
+                logging.info("Running unpack_magic on %s", filename)
+                unpack_error, newfiles = unpack_magic(nzo, workdir, tmp_workdir_complete, flag_delete, one_folder, (), (), (), (), ())
+                logging.info("Unpacked files %s", newfiles)
+
+                if sabnzbd.WIN32:
+                    # Sanitize the resulting files
+                    newfiles = sanitize_files_in_folder(tmp_workdir_complete)
+                logging.info("Finished unpack_magic on %s", filename)
 
             if cfg.safe_postproc():
                 all_ok = all_ok and not unpack_error
@@ -418,7 +421,7 @@ def process_job(nzo):
                 # Check if this is an NZB-only download, if so redirect to queue
                 # except when PP was Download-only
                 if flag_repair:
-                    nzb_list = nzb_redirect(tmp_workdir_complete, nzo.final_name, nzo.pp, script, cat, priority=nzo.priority)
+                    nzb_list = nzb_redirect(tmp_workdir_complete, nzo.final_name, nzo.pp, script, nzo.cat, priority=nzo.priority)
                 else:
                     nzb_list = None
                 if nzb_list:
@@ -443,7 +446,6 @@ def process_job(nzo):
                 else:
                     workdir_complete = tmp_workdir_complete.replace('_UNPACK_', '_FAILED_')
                     workdir_complete = get_unique_path(workdir_complete, n=0, create_dir=False)
-                    workdir_complete = workdir_complete
 
             if empty:
                 job_result = -1
@@ -467,18 +469,12 @@ def process_job(nzo):
             # Run the user script
             script_path = make_script_path(script)
             if (all_ok or not cfg.safe_postproc()) and (not nzb_list) and script_path:
-                # For windows, we use Short-Paths until 2.0.0 for compatibility
-                if sabnzbd.WIN32 and len(workdir_complete) > 259:
-                    import win32api
-                    workdir_complete = win32api.GetShortPathName(workdir_complete)
-
-                # set the current nzo status to "Ext Script...". Used in History
+                # Set the current nzo status to "Ext Script...". Used in History
                 nzo.status = Status.RUNNING
                 nzo.set_action_line(T('Running script'), unicoder(script))
                 nzo.set_unpack_info('Script', T('Running user script %s') % unicoder(script), unique=True)
-                script_log, script_ret = external_processing(script_path, workdir_complete, nzo.filename,
-                                                             dirname, cat, nzo.group, job_result,
-                                                             nzo.nzo_info.get('failure', ''))
+                script_log, script_ret = external_processing(script_path, nzo, clip_path(workdir_complete),
+                                                             nzo.final_name, job_result)
                 script_line = get_last_line(script_log)
                 if script_log:
                     script_output = nzo.nzo_id
@@ -502,7 +498,7 @@ def process_job(nzo):
         # Email the results
         if (not nzb_list) and cfg.email_endjob():
             if (cfg.email_endjob() == 1) or (cfg.email_endjob() == 2 and (unpack_error or par_error or script_error)):
-                emailer.endjob(dirname, cat, all_ok, workdir_complete, nzo.bytes_downloaded,
+                emailer.endjob(nzo.final_name, nzo.cat, all_ok, workdir_complete, nzo.bytes_downloaded,
                                nzo.fail_msg, nzo.unpack_info, script, TRANS(script_log), script_ret)
 
         if script_output:
@@ -519,7 +515,6 @@ def process_job(nzo):
                 # No '(more)' button needed
                 nzo.set_unpack_info('Script', u'%s%s ' % (script_ret, script_line), unique=True)
 
-
         # Cleanup again, including NZB files
         if all_ok:
             cleanup_list(workdir_complete, False)
@@ -538,26 +533,18 @@ def process_job(nzo):
                 for host in hosts:
                     Rating.do.update_auto_flag(nzo.nzo_id, Rating.FLAG_EXPIRED, host)
 
-        # Show final status in history
-        if all_ok:
-            notifier.send_notification(T('Download Completed'), filename, 'complete')
-            nzo.status = Status.COMPLETED
-        else:
-            notifier.send_notification(T('Download Failed'), filename, 'failed')
-            nzo.status = Status.FAILED
-
     except:
-        logging.error(T('Post Processing Failed for %s (%s)'), filename, crash_msg)
-        if not crash_msg:
-            logging.info("Traceback: ", exc_info=True)
-            crash_msg = T('see logfile')
-        nzo.fail_msg = T('PostProcessing was aborted (%s)') % unicoder(crash_msg)
-        notifier.send_notification(T('Download Failed'), filename, 'failed')
+        logging.error(T('Post Processing Failed for %s (%s)'), filename, T('see logfile'))
+        logging.info("Traceback: ", exc_info=True)
+
+        nzo.fail_msg = T('PostProcessing was aborted (%s)') % T('see logfile')
+        notifier.send_notification(T('Download Failed'), filename, 'failed', nzo.cat)
         nzo.status = Status.FAILED
         par_error = True
         all_ok = False
+
         if cfg.email_endjob():
-            emailer.endjob(dirname, cat, all_ok, clip_path(workdir_complete), nzo.bytes_downloaded,
+            emailer.endjob(nzo.final_name, nzo.cat, all_ok, clip_path(workdir_complete), nzo.bytes_downloaded,
                            nzo.fail_msg, nzo.unpack_info, '', '', 0)
 
     if all_ok:
@@ -566,17 +553,6 @@ def process_job(nzo):
         workdir_complete = one_file_or_folder(workdir_complete)
         workdir_complete = os.path.normpath(workdir_complete)
 
-    # Log the overall time taken for postprocessing
-    postproc_time = int(time.time() - start)
-
-    # Create the history DB instance
-    history_db = database.HistoryDB()
-    # Add the nzo to the database. Only the path, script and time taken is passed
-    # Other information is obtained from the nzo
-    history_db.add_history_db(nzo, clip_path(workdir_complete), nzo.downpath, postproc_time, script_log, script_line)
-    # The connection is only used once, so close it here
-    history_db.close()
-
     # Clean up the NZO
     try:
         logging.info('Cleaning up %s (keep_basic=%s)', filename, str(not all_ok))
@@ -599,40 +575,93 @@ def process_job(nzo):
     if par_error or unpack_error in (2, 3):
         try_alt_nzb(nzo)
 
-    # Update the last check time
-    sabnzbd.LAST_HISTORY_UPDATE = time.time()
+    # Show final status in history
+    if all_ok:
+        notifier.send_notification(T('Download Completed'), filename, 'complete', nzo.cat)
+        nzo.status = Status.COMPLETED
+    else:
+        notifier.send_notification(T('Download Failed'), filename, 'failed', nzo.cat)
+        nzo.status = Status.FAILED
 
+    # Log the overall time taken for postprocessing
+    postproc_time = int(time.time() - start)
+
+    # Create the history DB instance
+    history_db = database.HistoryDB()
+    # Add the nzo to the database. Only the path, script and time taken is passed
+    # Other information is obtained from the nzo
+    history_db.add_history_db(nzo, clip_path(workdir_complete), nzo.downpath, postproc_time, script_log, script_line)
+    # Purge items
+    history_db.auto_history_purge()
+    # The connection is only used once, so close it here
+    history_db.close()
+    sabnzbd.history_updated()
     return True
 
 
-def is_parfile(fn):
-    """ Check quickly whether file has par2 signature """
-    PAR_ID = "PAR2\x00PKT"
-    try:
-        with open(fn, "rb") as f:
-            buf = f.read(8)
-            return buf.startswith(PAR_ID)
-    except:
-        pass
-    return False
+def prepare_extraction_path(nzo):
+    """ Based on the information that we have, generate
+        the extraction path and create the directory.
+        Separated so it can be called from DirectUnpacker
+    """
+    one_folder = False
+    marker_file = None
+    # Determine class directory
+    catdir = config.get_categories(nzo.cat).dir()
+    if catdir.endswith('*'):
+        catdir = catdir.strip('*')
+        one_folder = True
+    complete_dir = real_path(cfg.complete_dir.get_path(), catdir)
+    complete_dir = long_path(complete_dir)
+
+    # TV/Movie/Date Renaming code part 1 - detect and construct paths
+    if cfg.enable_meta():
+        file_sorter = Sorter(nzo, nzo.cat)
+    else:
+        file_sorter = Sorter(None, nzo.cat)
+    complete_dir = file_sorter.detect(nzo.final_name, complete_dir)
+    if file_sorter.sort_file:
+        one_folder = False
+
+    complete_dir = sanitize_and_trim_path(complete_dir)
+
+    if one_folder:
+        workdir_complete = create_dirs(complete_dir)
+    else:
+        workdir_complete = get_unique_path(os.path.join(complete_dir, nzo.final_name), create_dir=True)
+        marker_file = set_marker(workdir_complete)
+
+    if not workdir_complete or not os.path.exists(workdir_complete):
+        logging.error(T('Cannot create final folder %s') % unicoder(os.path.join(complete_dir, nzo.final_name)))
+        raise IOError
+
+    if cfg.folder_rename() and not one_folder:
+        prefixed_path = prefix(workdir_complete, '_UNPACK_')
+        tmp_workdir_complete = get_unique_path(prefix(workdir_complete, '_UNPACK_'), create_dir=False)
+
+        try:
+            renamer(workdir_complete, tmp_workdir_complete)
+        except:
+            pass  # On failure, just use the original name
+
+        # Is the unique path different? Then we also need to modify the final path
+        if prefixed_path != tmp_workdir_complete:
+            workdir_complete = workdir_complete + os.path.splitext(tmp_workdir_complete)[1]
+    else:
+        tmp_workdir_complete = workdir_complete
+
+    return tmp_workdir_complete, workdir_complete, file_sorter, one_folder, marker_file
 
 
 def parring(nzo, workdir):
     """ Perform par processing. Returns: (par_error, re_add) """
-    if 0: assert isinstance(nzo, sabnzbd.nzbstuff.NzbObject) # Assert only for debug purposes
     filename = nzo.final_name
-    notifier.send_notification(T('Post-processing'), filename, 'pp')
+    notifier.send_notification(T('Post-processing'), filename, 'pp', nzo.cat)
     logging.info('Starting verification and repair of %s', filename)
 
     # Get verification status of sets
     verified = sabnzbd.load_data(VERIFIED_FILE, nzo.workpath, remove=False) or {}
-
-    # Collect the par files
-    if nzo.partable:
-        par_table = nzo.partable.copy()
-    else:
-        par_table = {}
-    repair_sets = par_table.keys()
+    repair_sets = nzo.extrapars.keys()
 
     re_add = False
     par_error = False
@@ -640,12 +669,14 @@ def parring(nzo, workdir):
 
     if repair_sets:
         for setname in repair_sets:
-            if cfg.ignore_samples() and 'sample' in setname.lower():
+            if cfg.ignore_samples() and RE_SAMPLE.search(setname.lower()):
                 continue
             if not verified.get(setname, False):
                 logging.info("Running verification and repair on set %s", setname)
-                parfile_nzf = par_table[setname]
-                if os.path.exists(os.path.join(nzo.downpath, parfile_nzf.filename)) or parfile_nzf.extrapars:
+                parfile_nzf = nzo.partable[setname]
+
+                # Check if file maybe wasn't deleted and if we maybe have more files in the parset
+                if os.path.exists(os.path.join(nzo.downpath, parfile_nzf.filename)) or nzo.extrapars[setname]:
                     need_re_add, res = par2_repair(parfile_nzf, nzo, workdir, setname, single=single)
 
                     # Was it aborted?
@@ -655,64 +686,29 @@ def parring(nzo, workdir):
                         break
 
                     re_add = re_add or need_re_add
-                    if not res and not need_re_add and cfg.sfv_check():
-                        res = try_sfv_check(nzo, workdir, setname)
-                    if not res and not need_re_add and cfg.enable_unrar():
-                        res = try_rar_check(nzo, workdir, setname)
                     verified[setname] = res
                 else:
                     continue
                 par_error = par_error or not res
+
     else:
-        # Obfuscated par2 check
-        logging.info('No par2 sets found, running obfuscated check on %s', filename)
+        # We must not have found any par2..
+        logging.info("No par2 sets for %s", filename)
+        nzo.set_unpack_info('Repair', T('[%s] No par2 sets') % unicoder(filename))
+        if cfg.sfv_check() and not verified.get('', False):
+            par_error = not try_sfv_check(nzo, workdir, '')
+            verified[''] = not par_error
+        # If still no success, do RAR-check
+        if not par_error and cfg.enable_unrar():
+            par_error = not try_rar_check(nzo, workdir, '')
+            verified[''] = not par_error
 
-        # Get the NZF's and sort them based on size
-        nzfs_sorted = sorted(nzo.finished_files, key=lambda x: x.bytes)
-
-        # We will have to make 'fake' par files that are recognized
-        par2_vol = 0
-        par2_filename = None
-
-        for nzf_try in nzfs_sorted:
-            # run through list of files, looking for par2 signature..
-            logging.debug("Checking par2 signature of %s", nzf_try.filename)
-            try:
-                nzf_path = os.path.join(workdir, nzf_try.filename)
-                if(is_parfile(nzf_path)):
-                    # We need 1 base-name so they are recognized as 1 set
-                    if not par2_filename:
-                        par2_filename = nzf_path
-
-                    # Rename so handle_par2() picks it up
-                    newpath = '%s.vol%d+%d.par2' % (par2_filename, par2_vol, par2_vol+1)
-                    renamer(nzf_path, newpath)
-                    nzf_try.filename = os.path.split(newpath)[1]
-
-                    # Let the magic happen
-                    nzo.handle_par2(nzf_try, file_done=True)
-                    par2_vol += 1
-            except:
-                pass
-        if par2_vol > 0:
-            # Pars found, we do it again
-            par_error, re_add = parring(nzo, workdir)
-        else:
-            # We must not have found any par2..
-            logging.info("No par2 sets for %s", filename)
-            nzo.set_unpack_info('Repair', T('[%s] No par2 sets') % unicoder(filename))
-            if cfg.sfv_check() and not verified.get('', False):
-                par_error = not try_sfv_check(nzo, workdir, '')
-                verified[''] = not par_error
-            # If still no success, do RAR-check
-            if not par_error and cfg.enable_unrar():
-                par_error = not try_rar_check(nzo, workdir, '')
-                verified[''] = not par_error
     if re_add:
         logging.info('Re-added %s to queue', filename)
         if nzo.priority != TOP_PRIORITY:
             nzo.priority = REPAIR_PRIORITY
-        sabnzbd.nzbqueue.add_nzo(nzo)
+        nzo.status = Status.FETCHING
+        sabnzbd.nzbqueue.NzbQueue.do.add(nzo)
         sabnzbd.downloader.Downloader.do.resume_from_postproc()
 
     sabnzbd.save_data(verified, VERIFIED_FILE, nzo.workpath)
@@ -775,7 +771,7 @@ def try_rar_check(nzo, workdir, setname):
     # Test
     if rars:
         nzo.status = Status.VERIFYING
-        nzo.set_unpack_info('Repair', T('Trying RAR-based verification'), set=setname)
+        nzo.set_unpack_info('Repair', T('Trying RAR-based verification'))
         nzo.set_action_line(T('Trying RAR-based verification'), '...')
         try:
             # Set path to unrar and open the file
@@ -786,20 +782,20 @@ def try_rar_check(nzo, workdir, setname):
             # Skip if it's encrypted
             if zf.needs_password():
                 msg = T('[%s] RAR-based verification failed: %s') % (unicoder(os.path.basename(rars[0])), T('Passworded'))
-                nzo.set_unpack_info('Repair', msg, set=setname)
+                nzo.set_unpack_info('Repair', msg)
                 return True
 
             # Will throw exception if something is wrong
             zf.testrar()
             # Success!
             msg = T('RAR files verified successfully')
-            nzo.set_unpack_info('Repair', msg, set=setname)
+            nzo.set_unpack_info('Repair', msg)
             logging.info(msg)
             return True
         except rarfile.Error as e:
             nzo.fail_msg = T('RAR files failed to verify')
-            msg = T('[%s] RAR-based verification failed: %s') % (unicoder(os.path.basename(rars[0])), unicoder(e.message.replace('\r\n',' ')))
-            nzo.set_unpack_info('Repair', msg, set=setname)
+            msg = T('[%s] RAR-based verification failed: %s') % (unicoder(os.path.basename(rars[0])), unicoder(e.message.replace('\r\n', ' ')))
+            nzo.set_unpack_info('Repair', msg)
             logging.info(msg)
             return False
     else:
@@ -807,33 +803,20 @@ def try_rar_check(nzo, workdir, setname):
         return True
 
 
-def addPrefixes(path, dirprefix):
-    """ Add list of prefixes as sub folders to path
-        '/my/path' and ['a', 'b', 'c'] will give '/my/path/a/b/c'
-    """
-    for folder in dirprefix:
-        if not folder:
-            continue
-        if not path:
-            break
-        basepath = os.path.basename(os.path.abspath(path))
-        if folder != basepath.lower():
-            path = os.path.join(path, folder)
-    return path
-
-
 def handle_empty_queue():
     """ Check if empty queue calls for action """
     if sabnzbd.nzbqueue.NzbQueue.do.actives() == 0:
         sabnzbd.save_state()
-        logging.info("Queue has finished, launching: %s (%s)",
-                     sabnzbd.QUEUECOMPLETEACTION, sabnzbd.QUEUECOMPLETEARG)
-        if sabnzbd.QUEUECOMPLETEARG:
-            sabnzbd.QUEUECOMPLETEACTION(sabnzbd.QUEUECOMPLETEARG)
-        else:
-            Thread(target=sabnzbd.QUEUECOMPLETEACTION).start()
 
-        sabnzbd.change_queue_complete_action(cfg.queue_complete(), new=False)
+        # Perform end-of-queue action when one is set
+        if sabnzbd.QUEUECOMPLETEACTION:
+            logging.info("Queue has finished, launching: %s (%s)",
+                         sabnzbd.QUEUECOMPLETEACTION, sabnzbd.QUEUECOMPLETEARG)
+            if sabnzbd.QUEUECOMPLETEARG:
+                sabnzbd.QUEUECOMPLETEACTION(sabnzbd.QUEUECOMPLETEARG)
+            else:
+                Thread(target=sabnzbd.QUEUECOMPLETEACTION).start()
+            sabnzbd.change_queue_complete_action(cfg.queue_complete(), new=False)
 
 
 def cleanup_list(wdir, skip_nzb):
@@ -853,7 +836,7 @@ def cleanup_list(wdir, skip_nzb):
                 if on_cleanup_list(filename, skip_nzb):
                     try:
                         logging.info("Removing unwanted file %s", path)
-                        os.remove(path)
+                        remove_file(path)
                     except:
                         logging.error(T('Removing %s failed'), clip_path(path))
                         logging.info("Traceback: ", exc_info=True)
@@ -877,10 +860,7 @@ def nzb_redirect(wdir, nzbname, pp, script, cat, priority):
         if so send to queue and remove if on CleanList
         Returns list of processed NZB's
     """
-    files = []
-    for root, _dirs, names in os.walk(wdir):
-        for name in names:
-            files.append(os.path.join(root, name))
+    files = recursive_listdir(wdir)
 
     for file_ in files:
         if os.path.splitext(file_)[1].lower() != '.nzb':
@@ -900,10 +880,14 @@ def nzb_redirect(wdir, nzbname, pp, script, cat, priority):
 def one_file_or_folder(folder):
     """ If the dir only contains one file or folder, join that file/folder onto the path """
     if os.path.exists(folder) and os.path.isdir(folder):
-        cont = os.listdir(folder)
-        if len(cont) == 1:
-            folder = os.path.join(folder, cont[0])
-            folder = one_file_or_folder(folder)
+        try:
+            cont = os.listdir(folder)
+            if len(cont) == 1:
+                folder = os.path.join(folder, cont[0])
+                folder = one_file_or_folder(folder)
+        except WindowsError:
+            # Can occur on paths it doesn't like, for example "C:"
+            pass
     return folder
 
 
@@ -927,14 +911,13 @@ def get_last_line(txt):
 
 def remove_samples(path):
     """ Remove all files that match the sample pattern """
-    RE_SAMPLE = re.compile(sample_match, re.I)
     for root, _dirs, files in os.walk(path):
         for file_ in files:
             if RE_SAMPLE.search(file_):
                 path = os.path.join(root, file_)
                 try:
                     logging.info("Removing unwanted sample file %s", path)
-                    os.remove(path)
+                    remove_file(path)
                 except:
                     logging.error(T('Removing %s failed'), clip_path(path))
                     logging.info("Traceback: ", exc_info=True)
@@ -987,7 +970,7 @@ def del_marker(path):
     if path and os.path.exists(path):
         logging.debug('Removing marker file %s', path)
         try:
-            os.remove(path)
+            remove_file(path)
         except:
             logging.info('Cannot remove marker file %s', path)
             logging.info("Traceback: ", exc_info=True)
